# -*- coding: utf-8 -*-


from django.utils.translation import ugettext as _
from djangoplus.utils.metadata import set_metadata, iterable


def meta(verbose_name, help_text=None, formatter=None, dashboard=None, shortcut=False, can_view=(), expose=True, icon=None):
    def decorate(func):
        set_metadata(func, 'type', 'attr')
        set_metadata(func, 'verbose_name', verbose_name)
        set_metadata(func, 'can_view', can_view)
        set_metadata(func, 'help_text', help_text)
        set_metadata(func, 'formatter', formatter)
        set_metadata(func, 'dashboard', dashboard)
        set_metadata(func, 'shortcut', shortcut)
        set_metadata(func, 'icon', icon)
        set_metadata(func, 'expose', iterable(expose))
        return func
    return decorate


def subset(verbose_name, help_text=None, list_display=(), list_filter=None, search_fields=None, template=None, menu=None,
           dashboard=None, usecase=None, can_view=(), can_alert=(), can_notify=(), expose=True):
    def decorate(func):
        from djangoplus import next_number
        set_metadata(func, 'type', 'subset')
        set_metadata(func, 'tab', True)
        set_metadata(func, 'verbose_name', verbose_name)
        set_metadata(func, 'alert', can_alert)
        set_metadata(func, 'notify', can_notify)
        set_metadata(func, 'menu', menu)
        set_metadata(func, 'help_text', help_text)
        set_metadata(func, 'usecase', usecase)
        set_metadata(func, 'can_view', iterable(can_view))
        set_metadata(func, 'name', func.__name__)
        set_metadata(func, 'order', next_number())
        set_metadata(func, 'dashboard', dashboard)
        set_metadata(func, 'list_display', list_display)
        set_metadata(func, 'list_filter', list_filter)
        set_metadata(func, 'search_fields', search_fields)
        set_metadata(func, 'template', template)
        set_metadata(func, 'formatter', None)
        set_metadata(func, 'expose', iterable(expose))
        return func

    return decorate


def action(verbose_name, help_text=None, condition=None, inline=False, subset=(), icon=None, category=None,
           style='popup', message=_('Action successfully performed.'), redirect_to=None, menu=None, initial=None,
           choices=None, display=None, input=None, usecase=None, can_execute=(), can_execute_by_organization=None,
           can_execute_by_unit=None, can_execute_by_role=None, expose=True):
    def decorate(func):
        func._action = dict(
            verbose_name=verbose_name, can_execute=iterable(can_execute), help_text=help_text,
            input=input, group=category or verbose_name, style=style, condition=condition, view_name=func.__name__,
            message=message, initial=initial or '{}_initial'.format(func.__name__), function=func,
            choices=choices or '{}_choices'.format(func.__name__), inline=inline, subsets=iterable(subset), icon=icon,
            doc=func.__doc__, usecase=usecase, can_execute_by_organization=iterable(can_execute_by_organization),
            can_execute_by_unit=iterable(can_execute_by_unit), can_execute_by_role=iterable(can_execute_by_role),
            redirect_to=redirect_to, menu=menu, display=display, source='model', expose=iterable(expose)
        )
        return func

    return decorate


def role(username, email=None, name=None, active=None, scope=None, signup=False, notify=False):
    def decorate(cls):
        metaclass = getattr(cls, '_meta')
        metaclass.role_username = username
        metaclass.role_email = email
        metaclass.role_name = name
        metaclass.role_active = active
        metaclass.role_scope = scope
        metaclass.role_signup = signup
        metaclass.role_notify = notify
        return cls
    return decorate



# -*- coding: utf-8 -*-

import os
from djangoplus import test
from django.conf import settings


def parametrized(dec):
    def layer(*args, **kwargs):
        def repl(f):
            return dec(f, *args, **kwargs)
        return repl
    return layer


@parametrized
def testcase(func, title, username=settings.DEFAULT_SUPERUSER, password=settings.DEFAULT_PASSWORD, record=True):
    def wrapper(self):

        if username:
            if username != self.current_username:
                self.login(username, password)
            self.back()

        if test.CACHE['RECORD'] and record:
            print('Start recording {}'.format(func.__name__))
            self.recorder.start()
            if title:
                self.subtitle.display(title)

        func(self)

        if test.CACHE['RECORD'] and record:
            self.wait(6)
            output_dir = os.path.join(settings.BASE_DIR, 'videos')
            self.recorder.stop(title, output_dir, self.AUDIO_FILE_PATH)
            print('Stoped recording {}'.format(func.__name__))

        self.dump(func.__name__)

    test.CACHE['SEQUENCE'] += 1
    wrapper._sequence = test.CACHE['SEQUENCE']
    wrapper._funcname = func.__name__
    test.CACHE['RECORDING'] = False
    return wrapper



# -*- coding: utf-8 -*-
import os
import json
from django.conf import settings

DEBUG_EMAIL_FILE_PATH = os.path.join(settings.MEDIA_ROOT, 'mail.json')


def dump_emails(emails):
    messagens = []
    if not os.path.exists(settings.MEDIA_ROOT):
        os.mkdir(settings.MEDIA_ROOT)
    for message in emails:
        data = dict(
            from_email=message.from_email, to=', '.join(message.to), body=message.body,
            alternatives=message.alternatives
        )
        messagens.append(data)
    open(DEBUG_EMAIL_FILE_PATH, 'w').write(json.dumps(messagens))


def load_emails():
    messages = []
    if os.path.exists(DEBUG_EMAIL_FILE_PATH):
        messages = json.loads(open(DEBUG_EMAIL_FILE_PATH).read())
        os.unlink(DEBUG_EMAIL_FILE_PATH)
    return messages


def should_display():
    return os.path.exists(DEBUG_EMAIL_FILE_PATH)




# -*- coding: utf-8 -*-

from django.conf.urls import url
from djangoplus.ui.components.select import views

urlpatterns = [
    url(r'^autocomplete/(?P<app_name>\w+)/(?P<class_name>\w+)/$', views.autocomplete),
    url(r'^reload_options/(?P<app_name>\w+)/(?P<class_name>\w+)/(?P<current_value>\w+)/(?P<lookup>\w+)/(?P<selected_value>\w+)/(?P<lazy>\w+)/$', views.reload_options),
]



# -*- coding: utf-8 -*-

from django.core.management.base import BaseCommand
from djangoplus.admin.models import User, Role
from djangoplus.cache import CACHE


class Command(BaseCommand):
    def handle(self, *args, **options):
        Role.objects.all().delete()
        for cls in CACHE['ROLE_MODELS']:
            for o in cls.objects.all():
                o.save()
        User.objects.filter(role__group__isnull=True).exclude(is_superuser=True).delete()



# Generated by Django 2.1.1 on 2019-03-14 10:52

from django.db import migrations
import djangoplus.db.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ('admin', '0011_auto_20190312_1716'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='token',
            field=djangoplus.db.models.fields.CharField(max_length=255, null=True, verbose_name='Token'),
        ),
    ]



# -*- coding: utf-8 -*-

import json
from django.apps import apps
from django.http import HttpResponse
from django.views.decorators.csrf import csrf_exempt
from django.contrib.auth.decorators import login_required
from django.template.loader import render_to_string
from djangoplus.utils.serialization import dumps_qs_query, loads_qs_query
from djangoplus.utils.metadata import get_metadata


@login_required
@csrf_exempt
def autocomplete(request, app_name, class_name):
    results = []
    q = request.POST.get('q')
    qs = loads_qs_query(request.POST['qs[qs]'])
    search_fields = get_metadata(qs.model, 'search_fields', [])
    select_template = get_metadata(qs.model, 'select_template')
    select_display = get_metadata(qs.model, 'select_display')
    queryset = None
    if q:
        for i, search_field in enumerate(search_fields):
            if i == 0:
                queryset = qs.filter(**{'{}__icontains'.format(search_field): q})
            else:
                queryset = queryset | qs.filter(**{'{}__icontains'.format(search_field): q})
        if queryset is None:
            raise ValueError('The class {} does not have any search field.'.format(class_name))
    else:
        queryset = qs

    # queryset = queryset.all(request.user)
    for obj in queryset[0:25]:
        html = (select_template or select_display) and render_to_string(select_template or 'select_template.html', dict(obj=obj, select_display=select_display)) or str(obj)
        results.append(dict(id=obj.id, text=str(obj), html=html))
    s = json.dumps(dict(q=q, results=results))

    return HttpResponse(s)


@login_required
@csrf_exempt
def reload_options(request, app_name, class_name, current_value, lookup, selected_value, lazy):
    l = []
    pks = []
    if not current_value == '0':
        for pk in current_value.split('_'):
            pks.append(int(pk))
    selected_value = int(selected_value)
    lazy = int(lazy)
    cls = apps.get_model(app_name, class_name)
    select_template = get_metadata(cls, 'select_template')
    select_display = get_metadata(cls, 'select_display')
    queryset = cls.objects.filter(**{lookup: selected_value})

    data = dict(selected_value=selected_value, results=[], qs=lazy and dumps_qs_query(queryset) or None)
    if lazy:
        if pks:
            for obj in cls.objects.filter(pk__in=pks):
                html = (select_template or select_display) and render_to_string(select_template or 'select_template.html', dict(obj=obj, select_display=select_display)) or str(obj)
                data['results'].append(dict(id=obj.id, text=str(obj), html=html))
    else:
        for obj in queryset:
            html = (select_template or select_display) and render_to_string(select_template or 'select_template.html', dict(obj=obj, select_display=select_display)) or str(obj)
            data['results'].append(dict(id=obj.id, text=str(obj), html=html))
    s = json.dumps(data)

    return HttpResponse(s)


# -*- coding: utf-8 -*-

from django.apps import apps
from djangoplus.cache import CACHE
from django.contrib.auth.models import Permission, Group, ContentType
from django.db.models import signals
from django.contrib.auth.management import create_permissions
from django.contrib.auth import get_user_model
from djangoplus.utils.metadata import get_metadata


def sync_permissions():
    default_permissions_names = dict(edit='Editar', add='Cadastrar', delete='Excluir', list='Listar', view='Visualizar')
    get_user_model().objects.update(permission_mapping='{}')
    # Permission.objects.all().delete()
    # Group.objects.all().delete()

    for group in Group.objects.all():
        group.permissions.clear()

    for Model in apps.get_models():
        model_name = Model.__name__.lower()
        app_label = Model._meta.app_label

        if '_' not in model_name and not (model_name == 'user' and app_label == 'auth'):

            model = Model.__name__.lower()

            qs_content_type = ContentType.objects.filter(app_label=app_label, model=model)
            if qs_content_type.exists():
                content_type = qs_content_type[0]
            else:
                content_type = ContentType.objects.create(app_label=app_label, model=model)

            for codename, name in list(default_permissions_names.items()):
                codename = '{}_{}'.format(codename, content_type.model)
                name = '{} {}'.format(name, Model._meta.verbose_name)
                qs_permission = Permission.objects.filter(codename=codename, content_type=content_type)
                if qs_permission.exists():
                    qs_permission.update(name=name)
                else:
                    Permission.objects.create(codename=codename, name=name, content_type=content_type)
            permissions = []
            admin_groups = []
            for admin_group in get_metadata(Model, 'can_admin', (), iterable=True):
                admin_groups.append(admin_group)
            for scope in ('role', 'unit', 'organization'):
                for admin_group in get_metadata(Model, 'can_admin_by_{}'.format(scope), (), iterable=True):
                    admin_groups.append(admin_group)
            admin_groups = tuple(admin_groups)

            for codename, name, groups in get_metadata(Model, 'permissions', []):
                qs_permission = Permission.objects.filter(codename=codename, content_type=content_type)
                if qs_permission.exists():
                    qs_permission.update(name=name)
                else:
                    Permission.objects.create(codename=codename, name=name, content_type=content_type)
                if groups or admin_groups:
                    permissions.append((codename, admin_groups + groups))

            for codename in default_permissions_names:
                attr_name = 'can_{}'.format(codename)
                groups = []
                for group in get_metadata(Model, attr_name, (), iterable=True):
                    groups.append(group)
                for scope in ('role', 'unit', 'organization'):
                    for group in get_metadata(Model, '{}_by_{}'.format(attr_name, scope), (), iterable=True):
                        groups.append(group)
                groups = tuple(groups)
                if groups or admin_groups:
                    permissions.append((codename, admin_groups + groups))

            for codename, group_names in permissions:

                permissions = []

                if codename in default_permissions_names:
                    if codename == 'list':
                        permissions.append(Permission.objects.get(
                            codename='view_{}'.format(model), content_type__app_label=app_label
                        ))
                    permissions.append(Permission.objects.get(
                        codename='{}_{}'.format(codename, model), content_type__app_label=app_label
                    ))
                else:
                    permissions.append(Permission.objects.get(
                        codename='{}_{}'.format(codename, model), content_type__app_label=app_label
                    ))

                for permission in permissions:
                    for group_name in group_names:

                        group_name = hasattr(group_name, '_meta') and group_name._meta.verbose_name or group_name

                        if group_name in CACHE['ABSTRACT_ROLE_MODEL_NAMES']:
                            for concrete_group_name in CACHE['ABSTRACT_ROLE_MODEL_NAMES'][group_name]:
                                group = Group.objects.get_or_create(name=concrete_group_name.strip())[0]
                                group.permissions.add(permission)
                        else:
                            group = Group.objects.get_or_create(name=group_name.strip())[0]
                            group.permissions.add(permission)

    for view in CACHE['VIEWS']:
        func = view.get('func')
        if func:
            name = view.get('verbose_name')
            can_view = view.get('can_view', ())
            if can_view:
                content_type = ContentType.objects.get(app_label='admin', model='user')
                qs_permission = Permission.objects.filter(codename=func.__name__, content_type=content_type)
                if qs_permission.exists():
                    qs_permission.update(name=name)
                    permission = qs_permission[0]
                else:
                    permission = Permission.objects.create(codename=func.__name__, content_type=content_type, name=name)
                for group in Group.objects.filter(name__in=can_view):
                    group.permissions.add(permission)

    for actions_dict in (CACHE['INSTANCE_ACTIONS'], CACHE['QUERYSET_ACTIONS']):
        for model in actions_dict:
            app_label = get_metadata(model, 'app_label')
            content_type = ContentType.objects.get(app_label=app_label, model=model.__name__.lower())
            for category in actions_dict[model]:
                for key in list(actions_dict[model][category].keys()):
                    name = actions_dict[model][category][key]['verbose_name']
                    can_execute = []
                    for scope in ('', 'role', 'unit', 'organization'):
                        scope = scope and '_by_{}'.format(scope or scope)
                        for group_name in actions_dict[model][category][key].get('can_execute{}'.format(scope)) or ():
                            can_execute.append(group_name)
                    if can_execute:
                        qs_permission = Permission.objects.filter(codename=key, content_type=content_type)
                        if qs_permission.exists():
                            qs_permission.update(name=name)
                            permission = qs_permission[0]
                        else:
                            permission = Permission.objects.get_or_create(codename=key, content_type=content_type, name=name)[0]
                        for group in Group.objects.filter(name__in=can_execute):
                            group.permissions.add(permission)

signals.post_migrate.disconnect(create_permissions, dispatch_uid="django.contrib.auth.management.create_permissions")



# Generated by Django 2.1.7 on 2019-03-07 16:56

from django.conf import settings
from django.db import migrations
import django.db.models.deletion
import djangoplus.db.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ('admin', '0009_auto_20190212_1539'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='group',
            options={'verbose_name': 'Perfil', 'verbose_name_plural': 'Grupos'},
        ),
        migrations.AlterModelOptions(
            name='logindex',
            options={'verbose_name': 'Índices', 'verbose_name_plural': 'Índices'},
        ),
        migrations.AlterModelOptions(
            name='organization',
            options={'verbose_name': 'Organização', 'verbose_name_plural': 'Organizações'},
        ),
        migrations.AlterModelOptions(
            name='permission',
            options={'verbose_name': 'Permissão', 'verbose_name_plural': 'Permissões'},
        ),
        migrations.AlterModelOptions(
            name='role',
            options={'verbose_name': 'Função', 'verbose_name_plural': 'Funções'},
        ),
        migrations.AlterModelOptions(
            name='scope',
            options={'verbose_name': 'Escopo', 'verbose_name_plural': 'Escopos'},
        ),
        migrations.AlterModelOptions(
            name='settings',
            options={'verbose_name': 'Configurações', 'verbose_name_plural': 'Configurações'},
        ),
        migrations.AlterModelOptions(
            name='unit',
            options={'verbose_name': 'Unidade', 'verbose_name_plural': 'Unidades'},
        ),
        migrations.AlterModelOptions(
            name='user',
            options={'verbose_name': 'Login', 'verbose_name_plural': 'Usuários'},
        ),
        migrations.AlterField(
            model_name='log',
            name='content',
            field=djangoplus.db.models.fields.TextField(null=True, verbose_name='Conteúdo'),
        ),
        migrations.AlterField(
            model_name='log',
            name='content_type',
            field=djangoplus.db.models.fields.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='admin.ContentType', verbose_name='Objeto'),
        ),
        migrations.AlterField(
            model_name='log',
            name='date',
            field=djangoplus.db.models.fields.DateTimeField(auto_now=True, verbose_name='Data/Hora'),
        ),
        migrations.AlterField(
            model_name='log',
            name='object_description',
            field=djangoplus.db.models.fields.CharField(max_length=255, verbose_name='Descrição do Objeto'),
        ),
        migrations.AlterField(
            model_name='log',
            name='object_id',
            field=djangoplus.db.models.fields.IntegerField(verbose_name='Identificação'),
        ),
        migrations.AlterField(
            model_name='log',
            name='operation',
            field=djangoplus.db.models.fields.IntegerField(choices=[[1, 'Cadastrar'], [2, 'Editar'], [3, 'Excluir']], verbose_name='Operação'),
        ),
        migrations.AlterField(
            model_name='log',
            name='user',
            field=djangoplus.db.models.fields.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='Login'),
        ),
        migrations.AlterField(
            model_name='logindex',
            name='content_type',
            field=djangoplus.db.models.fields.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType', verbose_name='Objeto'),
        ),
        migrations.AlterField(
            model_name='logindex',
            name='object_id',
            field=djangoplus.db.models.fields.IntegerField(verbose_name='Identificação'),
        ),
        migrations.AlterField(
            model_name='role',
            name='active',
            field=djangoplus.db.models.fields.BooleanField(default=True, verbose_name='Ativo'),
        ),
        migrations.AlterField(
            model_name='role',
            name='group',
            field=djangoplus.db.models.fields.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='admin.Group', verbose_name='Perfil'),
        ),
        migrations.AlterField(
            model_name='role',
            name='scope',
            field=djangoplus.db.models.fields.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='admin.Scope', verbose_name='Escopo'),
        ),
        migrations.AlterField(
            model_name='role',
            name='user',
            field=djangoplus.db.models.fields.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='Login'),
        ),
        migrations.AlterField(
            model_name='settings',
            name='address',
            field=djangoplus.db.models.fields.TextField(blank=True, null=True, verbose_name='Endereço'),
        ),
        migrations.AlterField(
            model_name='settings',
            name='company',
            field=djangoplus.db.models.fields.CharField(blank=True, max_length=255, null=True, verbose_name='Nome da Empresa'),
        ),
        migrations.AlterField(
            model_name='settings',
            name='icon',
            field=djangoplus.db.models.fields.ImageField(blank=True, default='blank.png', null=True, upload_to='config', verbose_name='Ícone'),
        ),
        migrations.AlterField(
            model_name='settings',
            name='initials',
            field=djangoplus.db.models.fields.CharField(default='Django+', max_length=255, verbose_name='Nome'),
        ),
        migrations.AlterField(
            model_name='settings',
            name='logo',
            field=djangoplus.db.models.fields.ImageField(blank=True, default='', null=True, upload_to='config', verbose_name='Logotipo'),
        ),
        migrations.AlterField(
            model_name='settings',
            name='logo_pdf',
            field=djangoplus.db.models.fields.ImageField(blank=True, default='', help_text='Imagem sem plano de fundo', null=True, upload_to='config', verbose_name='Logotipo para PDF'),
        ),
        migrations.AlterField(
            model_name='settings',
            name='name',
            field=djangoplus.db.models.fields.CharField(default='Django Plus', max_length=255, verbose_name='Descrição'),
        ),
        migrations.AlterField(
            model_name='settings',
            name='phone_1',
            field=djangoplus.db.models.fields.PhoneField(blank=True, max_length=255, null=True, verbose_name='Telefone Principal'),
        ),
        migrations.AlterField(
            model_name='settings',
            name='phone_2',
            field=djangoplus.db.models.fields.PhoneField(blank=True, max_length=255, null=True, verbose_name='Telefone Secundário'),
        ),
        migrations.AlterField(
            model_name='settings',
            name='server_address',
            field=djangoplus.db.models.fields.CharField(default='http://localhost:8000', max_length=255, verbose_name='URL do Servidor'),
        ),
        migrations.AlterField(
            model_name='settings',
            name='system_email_address',
            field=djangoplus.db.models.fields.CharField(default='no-reply@djangoplus.net', max_length=255, verbose_name='E-amil de Notificação'),
        ),
        migrations.AlterField(
            model_name='settings',
            name='version',
            field=djangoplus.db.models.fields.CharField(max_length=255, null=True, verbose_name='Versão do Sistema'),
        ),
        migrations.AlterField(
            model_name='user',
            name='active',
            field=djangoplus.db.models.fields.BooleanField(default=True, verbose_name='Ativo'),
        ),
        migrations.AlterField(
            model_name='user',
            name='name',
            field=djangoplus.db.models.fields.CharField(blank=True, max_length=30, verbose_name='Nome'),
        ),
        migrations.AlterField(
            model_name='user',
            name='permission_mapping',
            field=djangoplus.db.models.fields.JsonField(null=True, verbose_name='Mapeamento de Permissões'),
        ),
        migrations.AlterField(
            model_name='user',
            name='photo',
            field=djangoplus.db.models.fields.ImageField(blank=True, default='user.png', null=True, upload_to='profiles', verbose_name='Foto'),
        ),
        migrations.AlterField(
            model_name='user',
            name='username',
            field=djangoplus.db.models.fields.CharField(max_length=30, unique=True, verbose_name='Login'),
        ),
    ]



# -*- coding: utf-8 -*-

import json
from os import path
from djangoplus import test
from django.conf import settings
from djangoplus.docs.diagrams import Workflow
from djangoplus.docs.usecase import UseCase
from django.core.management.commands.test import Command as DjangoTestComand


class Command(DjangoTestComand):

    def add_arguments(self, parser):
        super(Command, self).add_arguments(parser)
        parser.add_argument('--resume', action='store_true', dest='resume', default=False,
                            help='Resumes from the last successfull testcase')
        parser.add_argument('--continue', action='store', nargs=1, dest='continue', default=False,
                            help='Continues from a specified testcase')
        parser.add_argument('--testcase', action='store', nargs=1, dest='testcase', default=False,
                            help='Executes a specified testcase')
        parser.add_argument('--generate', action='store_true', dest='generate', default=(),
                            help='Adds test cases in test.py file')
        parser.add_argument('--watch', action='store_true', dest='watch', default=False,
                            help='Run tests in browser instead of headless mode')
        parser.add_argument('--record', action='store_true', dest='record', default=False,
                            help='Record a video for the testcase as a tutorial')
        parser.add_argument('--pause', action='store_true', dest='pause', default=False,
                            help='Pauses after the execution of the last testcase')

        parser.add_argument('--usecases', action='store_true', dest='usecases', default=False,
                            help='Lists the available testcases')
        parser.add_argument('--usecase', action='store', nargs=1, dest='usecase', default=False,
                            help='Generates the code for a specific testcase')

    def handle(self, *args, **options):
        watch = options.pop('watch', False)
        record = options.pop('record', False)
        upload = options.pop('upload', False)
        pause = options.pop('pause', False)

        if not watch and not record:
            test.CACHE['HEADLESS'] = True
        if record or upload:
            test.CACHE['RECORD'] = True
        test.CACHE['PAUSE'] = pause

        if options.pop('generate', False):
            workflow = Workflow()
            test_file_path = path.join(settings.BASE_DIR, settings.PROJECT_NAME, 'tests.py')
            code_list = []
            file_content = open(test_file_path).read()
            try:
                for task in workflow.tasks:
                    usecase = UseCase(task)
                    code = usecase.get_test_function_code()
                    if code and usecase.get_test_function_signature() not in file_content:
                        code_list.append(code)
                if code_list:
                    new_file_content = '{}\n{}\n'.format(file_content, '\n\n'.join(code_list))
                    open(test_file_path, 'w').write(new_file_content)
                    print('Testcases succesfully generanted into file "{}/tests.py"'.format(settings.PROJECT_NAME))
            except ZeroDivisionError as e:
                print(e)

        else:
            if options.pop('resume', False):
                file_path = '/tmp/{}.test'.format(settings.PROJECT_NAME)
                file_content = path.exists(file_path) and open(file_path).read() or ''
                if file_content:
                    data = json.loads(file_content)
                    test.CACHE['RESUME'] = data.get('step', None)
                    print('Resuming from step #{}'.format(test.CACHE['RESUME']))
            elif options.get('testcase', False):
                test.CACHE['STEP'] = options.get('testcase')[0]
                print('Executing step #{}'.format(test.CACHE['STEP']))
            elif options.get('continue', False):
                test.CACHE['CONTINUE'] = options.get('continue')[0]
                print('Continuing from step #{}'.format(test.CACHE['CONTINUE']))
            super(Command, self).handle(*args, **options)



# -*- coding: utf-8 -*-

# Generated by Django 1.10 on 2016-10-05 15:36


from django.db import migrations
import djangoplus.db.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ('admin', '0004_auto_20161001_1750'),
    ]

    operations = [
        migrations.AlterField(
            model_name='settings',
            name='theme',
            field=djangoplus.db.models.fields.CharField(choices=[[b'skin-7', b'skin-7'], [b'skin-6', b'skin-6'], [b'skin-5', b'skin-5'], [b'skin-4', b'skin-4'], [b'skin-3', b'skin-3'], [b'skin-2', b'skin-2'], [b'skin-1', b'skin-1']], default=b'skin-6', max_length=255, verbose_name='Tema'),
        ),
    ]



# -*- coding: utf-8 -*-
import os
import shutil
import djangoplus
from django.core.management.base import BaseCommand
from fabric.api import local, execute, lcd

password = os.environ.get('TWINE_PASSWORD')
basedir = os.path.dirname(os.path.realpath(os.path.dirname(djangoplus.__file__)))


def release(upload=True):

    with lcd(basedir):
        version = None
        new_version = None
        setup_file_path = os.path.join(basedir, 'setup.py')
        setup_file_lines = open(setup_file_path).readlines()
        for i, line in enumerate(setup_file_lines):
            if 'version=' in line:
                version = line.strip()[-4:-2]
                new_version = str(int(version)+1)
                setup_file_lines[i] = setup_file_lines[i].replace(version, new_version)
                break
        if new_version:
            if upload:
                open(setup_file_path, 'w').write(str(''.join(setup_file_lines)))
            github_git_dir_path = os.path.join(basedir, 'djangoplus/.git')
            github_git_tmp_path = '/tmp/djangoplus-version-{}'.format(version)
            print(version, '>>>', new_version)
            print('Moving', github_git_dir_path, github_git_tmp_path)
            shutil.move(github_git_dir_path, github_git_tmp_path)
            local('python setup.py sdist')
            print('Moving', github_git_tmp_path, github_git_dir_path)
            shutil.move(github_git_tmp_path, github_git_dir_path)
            if upload:
                local('twine upload dist/djangoplus-0.0.{}.tar.gz'.format(new_version))
                return new_version
            return version


def fake_release():
    version = release(False)
    with lcd(basedir):
        local('scp dist/djangoplus-0.0.{}.tar.gz root@djangoplus.net:/var/www/html/'.format(version))
        local('ssh root@djangoplus.net "chown www-data.www-data /var/www/html/djangoplus-0.0.{}.tar.gz"'.format(version))
        print('\n\n\nhttp://djangoplus.net/djangoplus-0.0.{}.tar.gz'.format(version))


class Command(BaseCommand):

    def add_arguments(self, parser):
        super(Command, self).add_arguments(parser)

        parser.add_argument('--fake', action='store_true', dest='fake', default=False,
                            help='Upload to djangoplus.net instead of pypi.org')

    def handle(self, *args, **options):
        execute_task = release
        if options.get('fake'):
            execute_task = fake_release
        execute(execute_task)




# Generated by Django 2.1.7 on 2019-04-14 10:49

from django.db import migrations
import django.db.models.deletion
import djangoplus.db.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ('admin', '0013_auto_20190326_2256'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='scope',
            field=djangoplus.db.models.fields.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='admin.Scope', verbose_name='Scope'),
        ),
        migrations.AlterField(
            model_name='settings',
            name='icon',
            field=djangoplus.db.models.fields.ImageField(blank=True, default=None, null=True, upload_to='config', verbose_name='Ícone'),
        ),
    ]



# -*- coding: utf-8 -*-

import six
import time
import hashlib
import os.path
import itertools
import datetime
from io import StringIO
from django.conf import settings
from dropbox import Dropbox, files
from django.core.files import File
from django.core.cache import cache
from dropbox.exceptions import ApiError
from django.core.files.storage import Storage
from django.utils.encoding import filepath_to_uri
from django.utils.deconstruct import deconstructible
from dropbox.files import FolderMetadata, FileMetadata

"""
Usage:
    document = models.FileField(
        verbose_name='Document',
        upload_to='dropbox:documents'
    )
"""


@deconstructible
class DropboxStorage(Storage):

    CHUNK_SIZE = 1 * 1024 * (settings.DEBUG and 1 or 1024)  # 1 Kb or 1Mb

    def __new__(cls, *args, **kwargs):
        if not hasattr(cls, '_instance'):
            cls._instance = super(DropboxStorage, cls).__new__(cls, *args, **kwargs)
        return cls._instance

    def __init__(self, location='/'):
        self.client = None
        self.location = location
        self.base_url = 'https://dl.dropboxusercontent.com/'
        self.base_dir = settings.MEDIA_ROOT
        self.tmp_dir = os.path.join(settings.BASE_DIR, '.dropbox')
        self.log_file = os.path.join(settings.BASE_DIR, '.dropbox.log')
        if not os.path.exists(self.tmp_dir):
            os.mkdir(self.tmp_dir)
        if not os.path.exists(self.log_file):
            file = open(self.log_file, 'w')
            file.close()

    def _get_client(self):
        if not self.client:
            self.client = Dropbox(settings.DROPBOX_TOKEN)
        return self.client

    def _get_abs_path(self, name):
        return os.path.realpath(os.path.join(self.location, name))

    def _open(self, name, mode='rb'):
        local_file_path = os.path.join(self.tmp_dir, name)
        if os.path.exists(local_file_path):
            return File(open(local_file_path, mode))
        else:
            return DropboxFile(self._get_abs_path(name), self, mode=mode)

    def _save(self, name, content):
        name = self._get_abs_path(name)
        relative_path = self._get_abs_path(name)[1:]
        file_path = os.path.join(self.base_dir, relative_path)
        dir_path = os.path.dirname(file_path)
        if not os.path.exists(dir_path):
            os.makedirs(dir_path)
        link_path = os.path.join(self.tmp_dir, relative_path)
        dir_path = os.path.dirname(link_path)
        if not os.path.exists(dir_path):
            os.makedirs(dir_path)
        with open(file_path, 'wb') as f:
            f.write(content.read())
            f.close()
        os.symlink(file_path, link_path)
        return name

    def _print_progress(self, iteration, total, prefix='', suffix='', decimals=1, length=100, fill='█'):
        percent = ("{0:." + str(decimals) + "f}").format(100 * (iteration / float(total)))
        filled = int(length * iteration // total)
        bar = fill * filled + '-' * (length - filled)
        if settings.DEBUG and 0:
            print('\r%s |%s| %s%% %s' % (prefix, bar, percent, suffix), end='\r')
        else:
            self._log('{}%'.format(percent))
        if iteration == total:
            if settings.DEBUG:
                print('')
            else:
                self._log('100%')

    def _log(self, info, sep='\n'):
        formated_time = datetime.datetime.now().strftime("[%d/%b/%Y %H:%M:%S]")
        if settings.DEBUG and 0:
            print('{} {}'.format(formated_time, info))
        else:
            file = open(self.log_file, 'a')
            file.write('{} {}'.format(formated_time, info))
            file.write(sep)
            file.close()

    def path(self, name):
        return os.path.join(self.base_dir, name)

    def delete(self, name):
        name = self._get_abs_path(name)
        self._get_client().files_delete_v2(name)

    def exists(self, name):
        try:
            self._get_client().files_get_metadata(name)
        except ApiError as e:
            if e.error.is_path() and e.error.get_path().is_not_found():
                return False
            raise e
        return True

    def exists_locally(self, name):
        return os.path.exists(os.path.join(self.base_dir, name))

    def listdir(self, path):
        path = self._get_abs_path(path)
        response = self._get_client().files_list_folder(path)
        directories = []
        files = []
        for entry in response.entries:
            if type(entry) == FolderMetadata:
                directories.append(os.path.basename(entry.path_display))
            elif type(entry) == FileMetadata:
                files.append(os.path.basename(entry.path_display))
        return directories, files

    def size(self, name):
        cache_key = 'django-dropbox-size:{}'.format(filepath_to_uri(name))
        size = cache.get(cache_key)
        if not size:
            size = self._get_client().files_get_metadata(name).size
            cache.set(cache_key, size, 3600 * 24 * 365)
        return size

    def url(self, name):
        return '/cloud/{}'.format(self._get_abs_path(name)[1:])

    def remote_url(self, name):
        cache_key = 'django-dropbox-size:{}'.format(filepath_to_uri(name))
        url = cache.get(cache_key)
        if not url:
            url = self._get_client().files_get_temporary_link(name).link
            cache.set(cache_key, url, 3600 * 3)
        return url

    def get_available_name(self, name, max_length=None):
        name = self._get_abs_path(name)
        dir_name, file_name = os.path.split(name)
        file_root, file_ext = os.path.splitext(file_name)
        count = itertools.count(1)
        while self.exists(name):
            name = os.path.join(dir_name, "%s_%s%s" % (file_root, next(count), file_ext))
        return name

    def sync(self):
        self._log('Starting sync on dir {}...'.format(self.tmp_dir))
        while True:
            for file_path in [os.path.join(dp, f) for dp, dn, names in os.walk(self.tmp_dir) for f in names]:
                local_hash = DropboxContentHasher.calculate(file_path)
                remote_path = file_path[len(self.tmp_dir):]
                if not self.exists(remote_path):
                    directory = os.path.dirname(remote_path)
                    if directory and not self.exists(directory):
                        self._get_client().files_create_folder_v2(directory)
                    with open(file_path, 'rb') as f:
                        file_size = os.path.getsize(file_path)
                        self._log('Uploading {} [{} bytes] ({}) to {}...'.format(
                            file_size, file_path, local_hash, remote_path)
                        )
                        if file_size <= self.CHUNK_SIZE:
                            self._get_client().files_upload(f.read(), remote_path)
                        else:
                            self._log('Starting session...')
                            upload_session_start_result = self._get_client().files_upload_session_start(
                                f.read(self.CHUNK_SIZE)
                            )
                            cursor = files.UploadSessionCursor(
                                session_id=upload_session_start_result.session_id,
                                offset=f.tell()
                            )
                            commit = files.CommitInfo(path=remote_path)
                            while f.tell() < file_size:
                                self._print_progress(f.tell(), file_size)
                                if file_size - f.tell() <= self.CHUNK_SIZE:
                                    self._get_client().files_upload_session_finish(
                                        f.read(self.CHUNK_SIZE), cursor, commit
                                    )
                                    self._print_progress(file_size, file_size)
                                    self._log('Session closed!')
                                else:
                                    self._get_client().files_upload_session_append_v2(
                                        f.read(self.CHUNK_SIZE), cursor
                                    )
                                    cursor.offset = f.tell()
                        self._log('Upload completed!')
                        local_hash = DropboxContentHasher.calculate(file_path)
                remote_hash = self._get_client().files_get_metadata(remote_path).content_hash
                self._log('Remote file found with hash {}!'.format(remote_hash))
                if local_hash == remote_hash:
                    self._log('Removing {}...'.format(file_path))
                    os.unlink(file_path)
                    self._log('File successfully uploaded: {} '.format(file_path))
            sleep_time = settings.DEBUG and 10 or 60*60
            self._log('Sleeping for {} seconds...'.format(sleep_time))
            time.sleep(sleep_time)


class DropboxFile(File):

    def __init__(self, name, storage, mode):
        self._storage = storage
        self._mode = mode
        self._is_dirty = False
        self.file = StringIO()
        self.start_range = 0
        self._name = name

    @property
    def size(self):
        if not hasattr(self, '_size'):
            self._size = self._storage.size(self._name)
        return self._size

    def read(self, num_bytes=None):
        metadata, response = self._storage.client.files_download(self._name)
        return response.content

    def write(self, content):
        if 'w' not in self._mode:
            raise AttributeError("File was opened for read-only access.")
        self.file = StringIO(content)
        self._is_dirty = True

    def close(self):
        if self._is_dirty:
            self._storage.client.files_upload(self.file.getvalue(), self._name)
        self.file.close()


class DropboxContentHasher(object):

    BLOCK_SIZE = 4 * 1024 * 1024

    def __init__(self):
        self._overall_hasher = hashlib.sha256()
        self._block_hasher = hashlib.sha256()
        self._block_pos = 0

        self.digest_size = self._overall_hasher.digest_size

    def update(self, new_data):
        if self._overall_hasher is None:
            raise AssertionError(
                "can't use this object anymore; you already called digest()")

        assert isinstance(new_data, six.binary_type), (
            "Expecting a byte string, got {!r}".format(new_data))

        new_data_pos = 0
        while new_data_pos < len(new_data):
            if self._block_pos == self.BLOCK_SIZE:
                self._overall_hasher.update(self._block_hasher.digest())
                self._block_hasher = hashlib.sha256()
                self._block_pos = 0

            space_in_block = self.BLOCK_SIZE - self._block_pos
            part = new_data[new_data_pos:(new_data_pos+space_in_block)]
            self._block_hasher.update(part)

            self._block_pos += len(part)
            new_data_pos += len(part)

    def _finish(self):
        if self._overall_hasher is None:
            raise AssertionError(
                "can't use this object anymore; you already called digest() or hexdigest()")

        if self._block_pos > 0:
            self._overall_hasher.update(self._block_hasher.digest())
            self._block_hasher = None
        h = self._overall_hasher
        self._overall_hasher = None
        return h

    def digest(self):
        return self._finish().digest()

    def hexdigest(self):
        return self._finish().hexdigest()

    def copy(self):
        c = DropboxContentHasher.__new__(DropboxContentHasher)
        c._overall_hasher = self._overall_hasher.copy()
        c._block_hasher = self._block_hasher.copy()
        c._block_pos = self._block_pos
        return c

    @staticmethod
    def calculate(file_path):
        hasher = DropboxContentHasher()
        with open(file_path, 'rb') as f:
            while True:
                chunk = f.read(1024)
                if len(chunk) == 0:
                    break
                hasher.update(chunk)
        return hasher.hexdigest()


class StreamHasher(object):

    def __init__(self, f, hasher):
        self._f = f
        self._hasher = hasher

    def close(self):
        return self._f.close()

    def flush(self):
        return self._f.flush()

    def fileno(self):
        return self._f.fileno()

    def tell(self):
        return self._f.tell()

    def read(self, *args):
        b = self._f.read(*args)
        self._hasher.update(b)
        return b

    def write(self, b):
        self._hasher.update(b)
        return self._f.write(b)

    def next(self):
        b = self._f.next()
        self._hasher.update(b)
        return b

    def readline(self, *args):
        b = self._f.readline(*args)
        self._hasher.update(b)
        return b

    def readlines(self, *args):
        b = None
        bs = self._f.readlines(*args)
        for b in bs:
            self._hasher.update(b)
        return b



# -*- coding: utf-8 -*-

from django.conf import settings
from importlib import import_module

modules = list()

modules.append(import_module('django.template.defaultfilters'))
for item in settings.TEMPLATES:
    for builtins in [
                'djangoplus.templatetags',
                'djangoplus.ui.components.paginator.templatetags'
            ]:
        modules.append(import_module(builtins))


def apply_filter(obj, filter_name, **kwargs):
    for module in modules:
        if hasattr(module, filter_name):
            func = getattr(module, filter_name)
            return func(obj, **kwargs)
    return obj



# -*- coding: utf-8 -*-

import re
import json
import locale
import qrcode
import base64
import tempfile
from uuid import uuid4

from django.conf import settings
from django.template.loader import render_to_string

from djangoplus.utils import http
from django.utils.safestring import mark_safe
from djangoplus.utils import formatter
from djangoplus.utils.metadata import get_metadata, getattr2


from django_jinja import library as register


@register.filter
def tojson(value):
    return json.dumps(value)


@register.filter
def short_username(value):
    return value.split('@')[0]


@register.filter
def displayable(value):
    return value not in [True, False, None, '']


@register.filter
def get_item(dictionary, key):
    return dictionary.get(key)


@register.global_function(name='uuid')
def do_uuid4():
    return uuid4()


@register.filter
def fieldset(obj, title, fields, actions=None, lookups=None):
    return mark_safe('{}|{}|{}|{}<br>'.format(title, fields, actions, lookups))


@register.filter
def actions(obj, action_list):
    return mark_safe('{}<br>'.format(action_list))


@register.filter
def full_date_format(value):
    locale.setlocale(locale.LC_ALL, "pt_BR")
    return value.strftime('%d de %B de %Y')


@register.filter(name='format', )
def format2(value, request=None):
    if value and hasattr(value, 'id') and hasattr(value, '_meta') and hasattr(value, 'fieldsets'):
        app = get_metadata(value.__class__, 'app_label')
        cls = value.__class__.__name__.lower()
        if request and request.user.has_perm('{}.{}'.format(app, cls)):
            return link(value)
    return formatter.format_value(value)


@register.filter
def print_format(value):
    return mark_safe(formatter.format_value(value).replace('\n', '<br />'))


@register.filter
def ordered_list(value):
    ol = ['<ol style="padding-left:13px">']
    for obj in value:
        ol.append('<li style="list-style-type:decimal">{}</li>'.format(obj))
    ol.append('</ol>')
    return mark_safe(''.join(ol))


@register.global_function
def value_at(col_value, col_index, template_filters):
    from . import utils
    value = col_value
    if col_index in template_filters:
        template_filter = template_filters[col_index]
        value = utils.apply_filter(value, template_filter)
    value = utils.apply_filter(value, 'format2')
    return value


@register.filter
def red_if_negative(obj):
    value = format2(obj)
    if obj < 0:
        return '<span class="text-danger">{}</span>'.format(value)
    return value


@register.filter
def children(obj):
    return getattr(obj, '{}_set'.format(obj.__class__.__name__.lower())).all()


@register.filter
def link(value):
    if value:
        app = get_metadata(value.__class__, 'app_label')
        cls = value.__class__.__name__.lower()

        return mark_safe('<a class="ajax" href="/view/{}/{}/{}/">{}</a>'.format(app, cls, value.pk, value))
    return '-'


@register.filter()
def mobile(request):
    return http.mobile(request)


@register.filter
def html(value):
    if value:
        value = str(value)
        value = value.replace('\n', '<br>').replace('  ', '&nbsp; ').replace('\t', '&nbsp;&nbsp;&nbsp;&nbsp;')
        return mark_safe(value)
    else:
        return '-'


@register.filter
def number(value):
    return str(value)


@register.filter
def tahoma(value):
    return mark_safe('<font style="font-family: tahoma">{}</font>'.format(html(value)))


@register.filter
def colspan(n):
    return int(12/n)


@register.filter
def must_hide_fieldset(tuples):
    count = 0
    for fields in tuples:
        count += len(fields)
    return count == 0


register.filter('getattr', getattr2)
register.filter('normalyze', formatter.normalyze)
register.filter('decimal1', formatter.format_decimal1)


@register.filter
def sorted_items(d):
    items = []
    keys = list(d.keys())
    keys.sort()
    for key in keys:
        items.append((re.sub('^[0-9\.\- ]+', '', key).strip(), d[key]))
    return items


@register.filter
def photo(user):
    if user and user.is_authenticated:
        if user.photo:
            return '/media/{}'.format(user.photo.name)
    return '/media/user.png'


@register.filter
def toast(m):
    params = "{{ text: '{}', loader: false, position : {{top: 60, right: 30}}, hideAfter: 10000}}".format(m.message)
    return mark_safe("<script>$.toast({});</script>".format(params))


@register.global_function
def set_request(obj, request):
    obj.request = request
    return ''


@register.filter
def is_image(value):
    if value:
        for ext in ('.png', '.jpg', '.jpeg', '.gif'):
            if str(value).lower().endswith(ext):
                return True
    return False


@register.filter
def image(value, attrs="width='100px'", zoom=False):
    return mark_safe('<img class="{}" {} src="{}">'.format(zoom and 'materialboxed' or '', attrs, image_url(value)))


@register.filter
def zoom_image(value, attrs="width='100px'"):
    return image(value, attrs=attrs, zoom=True)


@register.filter
def image_url(value):
    value = str(value)
    return '/static/' in value and value or '/media/{}'.format(value)


@register.filter
def qrcode64(text):
    qr = qrcode.QRCode()
    qr.add_data(text)
    image = qr.make_image()
    file_path = tempfile.mktemp()
    buffer = open(file_path, 'wb')
    image.save(buffer, format="JPEG")
    img_str = base64.b64encode(open(file_path, 'rb').read()).decode('utf-8')
    return mark_safe('<img width="100" src="data:image/jpeg;base64, {}"/>'.format(img_str))


@register.global_function
def captcha(form):
    return form.captcha and mark_safe('''
        <div align="center">
            <script src="https://www.google.com/recaptcha/api.js?hl={}"></script>
            <div style="width: 100%" class="g-recaptcha" data-sitekey="{}"></div>
        </div>
        <hr/>
    '''.format(settings.LANGUAGE_CODE, settings.CAPTCHA_KEY)) or ''


@register.filter
def snippet(obj, template_name):
    return render_to_string('{}.html'.format(template_name), dict(obj=obj))


@register.filter
def in_group(user, group_names):
    if user:
        return user.groups.filter(name__in=[name.strip() for name in group_names.split(',') if name]).exists()
    return False


@register.filter
def in_scope(obj, scope):
    from djangoplus.admin.models import User
    username_attr = get_metadata(obj, 'role_username')
    if username_attr:
        user = User.objects.get({username_attr: getattr(obj, username_attr)})
        return user.role_set.filter(scope=scope).exists()
    return False


@register.filter
def is_user(obj, request):
    username_attr = get_metadata(obj, 'role_username')
    if username_attr:
        return getattr(obj, username_attr) == request.user.username
    return False



# -*- coding: utf-8 -*-

from django.core.management.commands import runserver

from djangoplus.admin.models import User


class Command(runserver.Command):

    def handle(self, *args, **options):
        User.objects.update(permission_mapping='{}')
        super(Command, self).handle(*args, **options)



# -*- coding: utf-8 -*-

import os
from django.core.management import BaseCommand
from fabric.api import *


WORKSPACE_DIR = os.path.join(os.path.expanduser("~"), 'Documents/Workspace')


class Command(BaseCommand):

    def add_arguments(self, parser):
        super(Command, self).add_arguments(parser)

        parser.add_argument('--project', action='store_true', dest='project', default=False,
                            help='Test project creation')
        parser.add_argument('--admin', action='store_true', dest='admin', default=False,
                            help='Test admin application')
        parser.add_argument('--demo', action='store_true', dest='demo', default=False,
                            help='Test demo applications')
        parser.add_argument('--installation', action='store_true', dest='installation', default=False,
                            help='Test framework installation in debian/ubuntu')
        parser.add_argument('--implementation', action='store_true', dest='implementation', default=False,
                            help='Test framework implementation with several projects')

    def handle(self, *args, **options):
        execute_tasks = []
        if options.get('project'):
            execute_tasks.append(_test_startpoject)
        elif options.get('admin'):
            execute_tasks.append(_test_admin)
        elif options.get('demo'):
            execute_tasks.append(_test_demo_projects)
        elif options.get('installation'):
            execute_tasks.append(_test_so_installation)
        elif options.get('implementation'):
            execute_tasks.append(_test_projects)
        if not execute_tasks:
            execute_tasks = [_test_startpoject, _test_admin, _test_so_installation, _test_projects]
        for execute_task in execute_tasks:
            execute(execute_task)


EMPTY_TEST_FILE_CONTENT = '''# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from djangoplus._test_admin.models import User
from djangoplus.test import TestCase
from django.conf import settings


class AppTestCase(TestCase):

    def test_app(self):

        User.objects.create_superuser('_test_admin', None, settings.DEFAULT_PASSWORD)

        self.login('_test_admin', settings.DEFAULT_PASSWORD)
'''


DOCKER_FILE_CONTENT = '''FROM {}
ENV DEBIAN_FRONTEND=noninteractive
RUN apt-get update
RUN apt-get -y install python3 python3-pip build-essential python3-dev libfreetype6-dev python3-cffi libtiff5-dev liblcms2-dev libwebp-dev tk8.6-dev libjpeg-dev ssh openssh-server dnsutils curl vim git wget
RUN apt-get -y install chrpath libssl-dev libxft-dev libfreetype6 libfreetype6-dev libfontconfig1 libfontconfig1-dev
RUN apt-get -y install libgtk-3-dev

RUN curl https://ftp.mozilla.org/pub/firefox/releases/60.0b3/linux-x86_64/en-US/firefox-60.0b3.tar.bz2 --output firefox.tar.bz2
RUN tar xjf firefox.tar.bz2 --directory /usr/lib/
RUN ln -s /usr/lib/firefox/firefox /usr/local/bin/firefox
RUN rm firefox.tar.bz2

RUN curl http://djangoplus.net/geckodriver-v0.21.0-linux64.tar.gz --output geckodriver.tar.gz
RUN gunzip geckodriver.tar.gz
RUN tar -xf geckodriver.tar
RUN mv geckodriver /usr/local/bin/
RUN rm geckodriver.tar

ENV LC_ALL=C.UTF-8
ENV LANG=C.UTF-8
RUN export LANG=C.UTF-8

RUN ln -sfn /usr/bin/pip3 /usr/bin/pip
RUN ln -sfn /usr/bin/python3 /usr/bin/python

RUN pip install --upgrade pip
'''

DEMO_PROJECTS = [
    ('petshop', 'git@bitbucket.org/brenokcc/petshop.git'),
    ('loja', 'git@bitbucket.org/brenokcc/loja.git'),
    ('biblioteca', 'git@bitbucket.org/brenokcc/biblioteca.git'),
]

EXTERAL_PROJECTS = [
    ('companies', 'git@djangoplus.net:companies.git'),
    ('emprestimos', 'git@djangoplus.net:emprestimos.git'),
    ('financeiro', 'git@bitbucket.org:brenokcc/financeiro.git'),
    ('formulacao', 'git@bitbucket.org:brenokcc/formulacao.git'),
    ('gerifes', 'git@bitbucket.org/brenokcc/gerifes.git'),
    ('simop', 'git@bitbucket.org/brenokcc/simop.git'),
    ('sigplac', 'git@bitbucket.org:brenokcc/sigplac.git'),
    ('gouveia', 'git@bitbucket.org:brenokcc/gouveia.git'),
    ('abstract', 'git@bitbucket.org:brenokcc/abstract.git'),
    ('blackpoint', 'git@bitbucket.org:brenokcc/blackpoint.git'),
]


def _test_startpoject():
    django_settings_module = os.environ['DJANGO_SETTINGS_MODULE']
    os.environ['DJANGO_SETTINGS_MODULE'] = 'xxx.settings'
    if os.path.exists('/tmp/xxx'):
        local('rm -r /tmp/xxx')
    with lcd('/tmp/'):
        local('startproject xxx')
        with lcd('/tmp/xxx'):
            local('python manage.py test')
        with lcd('/tmp'):
            local('rm -r /tmp/xxx')
    os.environ['DJANGO_SETTINGS_MODULE'] = django_settings_module


def _test_admin():
    local('python manage.py test djangoplus.admin.tests.AdminTestCase')


def _test_demo_projects():
    return _test_projects(external=False)


def _test_projects(demo=True, external=True):
    paths = []
    os.environ['PYTHONIOENCODING'] = 'utf-8'
    projects = []
    if demo:
        projects += DEMO_PROJECTS
    if external:
        projects += EXTERAL_PROJECTS
    for project_name, project_url in projects:
        if os.path.exists(WORKSPACE_DIR):
            base_path = WORKSPACE_DIR
            if project_name in ('petshop', 'loja', 'biblioteca'):
                base_path = os.path.join(base_path, 'djangoplus/djangoplus-demos')
            project_path = os.path.join(base_path, project_name)
        else:
            project_path = os.path.join('/tmp', project_name)
            if not os.path.exists(project_path):
                local('git clone {} {}'.format(project_url, project_path))
            with lcd(project_path):
                local('git pull origin master')
        paths.append(project_path)

    django_settings_module = os.environ['DJANGO_SETTINGS_MODULE']
    for project_path in paths:
        project_name = project_path.split('/')[-1]
        print('Testing {}'.format(project_name))
        with lcd(project_path):
            os.environ['DJANGO_SETTINGS_MODULE'] = '{}.settings'.format(project_name)
            local('python manage.py test')
    os.environ['DJANGO_SETTINGS_MODULE'] = django_settings_module


def _test_testcases_generation():
    test_file_path = '{}/emprestimos/emprestimos/tests.py'.format(WORKSPACE_DIR)
    test_file_content = open(test_file_path).read()
    open(test_file_path, 'w').write(EMPTY_TEST_FILE_CONTENT)
    with lcd('{}/emprestimos'.format(WORKSPACE_DIR)):
        local('python manage.py test --add')
    print(open(test_file_path).read())
    open(test_file_path, 'w').write(test_file_content)


def _test_so_installation():
    for so in ('debian', ):  # ubuntu
        docker_file = open('/tmp/Dockerfile', 'w')
        docker_file.write(DOCKER_FILE_CONTENT.format(so))
        docker_file.close()
        local('docker build -t djangoplus-{} /tmp'.format(so))
        django_settings_module = os.environ['DJANGO_SETTINGS_MODULE']
        os.environ['DJANGO_SETTINGS_MODULE'] = 'xyz.settings'
        local('docker run djangoplus-{} pip install djangoplus && startproject xyz && cd xyz && python manage.py test djangoplus.admin.tests.AdminTestCase'.format(so))
        os.environ['DJANGO_SETTINGS_MODULE'] = django_settings_module


def _test_deploy():
    pass



# -*- coding: utf-8 -*-
import os
import signal
from time import sleep
from subprocess import Popen, PIPE, DEVNULL
from djangoplus.tools.terminal import simulate_type


class HttpServer(object):
    def __init__(self, base_dir, verbose=True, python='python'):
        self.process = None
        self.base_dir = base_dir
        self.verbose = verbose
        self.python = python

    def start(self):
        command = 'cd {} && {} manage.py runserver'.format(self.base_dir, self.python)
        if self.verbose:
            os.system('clear')
            simulate_type('python manage.py runserver', shell=True)
            stdout = PIPE
        else:
            stdout = DEVNULL
        self.process = Popen(command, stdout=stdout, stderr=PIPE, shell=True, preexec_fn=os.setsid)
        sleep(5)

    def stop(self):
        if self.verbose:
            print('^C')
        if self.process:
            os.killpg(os.getpgid(self.process.pid), signal.SIGTERM)




# Generated by Django 2.1.7 on 2019-03-12 17:16

from django.conf import settings
from django.db import migrations
import django.db.models.deletion
import djangoplus.db.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ('admin', '0010_auto_20190307_1656'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='user',
            options={'verbose_name': 'Usuário', 'verbose_name_plural': 'Usuários'},
        ),
        migrations.AlterField(
            model_name='log',
            name='user',
            field=djangoplus.db.models.fields.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='Usuário'),
        ),
        migrations.AlterField(
            model_name='role',
            name='user',
            field=djangoplus.db.models.fields.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='Usuário'),
        ),
    ]



# -*- coding: utf-8 -*-
import sys
import json
import urllib
from django.core import signing
from django.contrib import auth
from django.conf import settings
from djangoplus.mail import utils
from djangoplus.cache import CACHE
from djangoplus.utils.icons import ICONS
from djangoplus.admin.models import Settings
from django.http.response import HttpResponse
from django.utils.translation import ugettext as _
from djangoplus.decorators.views import view, action
from djangoplus.ui.components.panel import AppDashboard
from djangoplus.admin.models import User, Unit, Organization
from djangoplus.ui.components.navigation.breadcrumbs import httprr
from djangoplus.admin.forms import ProfileForm, ChangePasswordForm, SettingsForm, LoginForm, RecoverPassowordForm


@view(_('Public'), login_required=False)
def public(request):
    app_settings = Settings.default()
    if not app_settings.background:
        return httprr(request, '/admin/')
    return locals()


@view(_('Main'), login_required=True)
def index(request):
    widget_panel = AppDashboard(request)
    return locals()


@view(_('System Access'), login_required=False, template='login/login.html')
def login(request, scope=None, organization=None, unit=None):
    auth.logout(request)
    can_register = CACHE['SIGNUP_MODEL'] is not None

    allow_social_login = 'test' not in sys.argv
    google_auth_key = settings.GOOGLE_AUTH_KEY
    google_auth_id = settings.GOOGLE_AUTH_ID
    facebook_auth_id = settings.FACEBOOK_AUTH_ID

    organization = organization and Organization.objects.get(pk=organization) or None
    unit = unit and Unit.objects.get(pk=unit) or None
    form = LoginForm(request, scope, organization, unit)
    if form.is_valid():
        return form.submit()
    return locals()


@view(_('Logout'), login_required=False)
def logout(request):
    url = '/'
    unit_id = request.session.get('unit_id', None)
    if unit_id:
        url = '/admin/login/{}/'.format(unit_id)
    auth.logout(request)
    return httprr(request, url, _('You have successfully logged out.'))


@view('404')
def error(request):
    return locals()


@view(_('Change Password'), login_required=False, template='login/password.html')
def password(request, pk, token):
    title = _('Change Password')
    user = User.objects.filter(pk=pk, password=signing.loads(token)).first()
    if user:
        form = ChangePasswordForm(request, instance=user)
        if form.is_valid():
            form.save()
            return httprr(request, '/admin/', _('Your password has been successfully changed.'))
    else:
        return httprr(request, '/', _('No records found'))
    return locals()


@view(_('Reset Password'), login_required=False, template='login/reset.html')
def reset(request):
    title = _('Reset Password')
    form = RecoverPassowordForm(request)
    if form.is_valid():
        msg = _('Click on the link sent to your e-mail to reset your password.')
        return httprr(request, '/admin/login/', msg)
    return locals()


@view(_('Register'), login_required=False)
def register(request, token=None, userid=None):
    from djangoplus.ui.components import forms
    from djangoplus.utils.metadata import get_metadata

    initial = {}
    username_field = get_metadata(CACHE['SIGNUP_MODEL'], 'role_username')
    email_field = get_metadata(CACHE['SIGNUP_MODEL'], 'role_email')
    name_field = get_metadata(CACHE['SIGNUP_MODEL'], 'role_name')

    if not CACHE['SIGNUP_MODEL']:
        return httprr(request, '/admin/login/', _('Sign-up not enabled.'))

    if token:
        if token and userid:
            url = 'https://graph.facebook.com/{}?fields=email,first_name,last_name&access_token={}'
            url = url.format(userid, token)
        elif token:
            url = 'https://www.googleapis.com/oauth2/v1/userinfo?alt=json&access_token={}'.format(token)
        data = json.loads(urllib.request.urlopen(url).read())
        qs = User.objects.filter(username=data['email'])
        if qs.exists():
            user = qs[0]
            auth.login(request, user)
            return httprr(request, '/admin/', _('You have successfully logged in.'))
        else:
            initial = {name_field: data.get('name'), username_field: data['email'], email_field: data['email']}

    class RegisterForm(forms.ModelForm):
        class Meta:
            model = CACHE['SIGNUP_MODEL']
            fields = get_metadata(CACHE['SIGNUP_MODEL'], 'form_fields', '__all__')
            exclude = get_metadata(CACHE['SIGNUP_MODEL'], 'exclude_fields', ())
            submit_label = _('Register')
            title = '{} {}'.format(_('Register'), get_metadata(CACHE['SIGNUP_MODEL'], 'verbose_name'))
            icon = get_metadata(CACHE['SIGNUP_MODEL'], 'icon', None)
            captcha = settings.CAPTCHA_KEY and settings.CAPTCHA_SECRET and 'test' not in sys.argv or False

    form = RegisterForm(request, initial=initial)
    form.fields[username_field].help_text = _('Used to access the system')

    save_instance = True
    for field_name in form.fields:
        if not initial.get(field_name):
            save_instance = False
    if save_instance:
        instance = CACHE['SIGNUP_MODEL']()
        for field_name in form.fields:
            setattr(instance, field_name, initial[field_name])
        instance.save()
        user = User.objects.get(username=initial[username_field])
        auth.login(request, user)
        return httprr(request, '/admin/', _('You were successfully registered.'))

    if form.is_valid():
        instance = form.save()
        extra = email_field and _('An e-mail will be sent to you as soon as your account is activated.') or ''
        if instance:
            user = User.objects.get(username=form.cleaned_data[username_field])
            auth.login(request, user)
            return httprr(request, '/admin/', _('User successfully registered.'))
        else:
            return httprr(request, '..', _('Click on the link sent to your e-mail to have you account activated.'))
    return locals()


@view(_('Profile'))
def profile(request):
    form = ProfileForm(request, instance=request.user)
    if form.is_valid():
        form.save()
        return httprr(request, '..', _('Profile successfully updated.'))
    return locals()


@view(_('Configure'))
def configure(request):
    if not request.user.is_superuser:
        return httprr(request, '/', _('You do not have permission to access this page!'), 'error')
    title = _('Settings')
    form = SettingsForm(request)
    if form.is_valid():
        form.save()
        return httprr(request, '..', _('Configuration successfully updated.'))
    return locals()


@view(_('Sidebar'))
def toggle_menu(request):
    if 'hidden_menu' in request.session and request.session['hidden_menu']:
        request.session['hidden_menu'] = False
    else:
        request.session['hidden_menu'] = True
    request.session.save()
    return HttpResponse()


@action(User, _('Login as User'), can_execute='Superuser')
def login_as(request, pk):
    auth.logout(request)
    user = User.objects.get(pk=pk)
    auth.login(request, user)
    return httprr(request, '/admin/', _('User successfully authenticated.'))


@view(_('View E-mails'), login_required=False)
def emails(request):
    messages = utils.load_emails()
    return locals()


@view(_('Icons'), login_required=False)
def icons(request):
    icon_names = ICONS
    return locals()






# -*- coding: utf-8 -*-
from djangoplus.ui.components import forms
from djangoplus.ui.components.utils import MultiScheduleTable, ScheduleTable


class ScheduleTableForm(forms.Form):
    values = forms.CharField(label='Values', widget=forms.widgets.HiddenInput())

    def __init__(self, *args, **kwargs):
        schedule = kwargs.get('initial', {}).pop('schedule', [])
        kwargs['initial']['values'] = ''
        super().__init__(*args, **kwargs)
        form_prefix = self.prefix and '{}-'.format(self.prefix) or None
        self.component = MultiScheduleTable(schedule, self.request, title='Horários', form_prefix=form_prefix)

    def clean_values(self):
        values = self.cleaned_data['values']
        cleaned_values = []
        if values:
            for value in values.split('|'):
                i, interval = value.split('::')
                cleaned_values.append((ScheduleTable.WEEK_DAYS[int(i)-1], interval))
        return cleaned_values



# -*- coding: utf-8 -*-

from django.apps import apps
from threading import Thread
from django.conf import settings
from djangoplus.utils.metadata import get_metadata, get_scope, get_can_execute, count_parameters_names


CACHE = dict(
    INITIALIZED=False,
    SETTINGS_INSTANCE=None,
    # USER INTERFACE
    VIEWS=[],
    WIDGETS=[],
    SUBSET_WIDGETS=[],
    MODEL_WIDGETS={},
    CARD_PANEL_MODELS=[],
    ICON_PANEL_MODELS=[],
    LIST_DASHBOARD=[],
    SUBSETS=dict(),
    MANAGER_METHODS=dict(),
    INSTANCE_METHODS=dict(),
    SIMPLE_MODELS=[],
    # ROLES
    ROLE_MODELS=dict(),
    ABSTRACT_ROLE_MODELS=dict(),
    ABSTRACT_ROLE_MODEL_NAMES=dict(),
    # ACTIONS
    INSTANCE_ACTIONS=dict(),
    QUERYSET_ACTIONS=dict(),
    CLASS_ACTIONS=dict(),
    CLASS_VIEW_ACTIONS=dict(),
    FIELDSET_ACTIONS=dict(),
    # DOCUMENTATION
    WORKFLOWS=dict(),
    CLASS_DIAGRAMS=dict(),
    COMPOSITION_FIELDS=dict(),
    COMPOSITION_RELATIONS=dict(),
    # ACCESS SCOPE
    ORGANIZATION_MODEL=None,
    UNIT_MODEL=None,
    SIGNUP_MODEL=None,
    PERMISSIONS_BY_SCOPE=dict(),
    # FORMATTERS
    FORMATTERS=dict(),
    # DOCUMENTATION
    LAST_AUTHENTICATED_ROLE=None,
    LAST_AUTHENTICATED_USERNAME=None,
    # API
    API_MODELS=[]
)


if not CACHE['INITIALIZED']:
    CACHE['INITIALIZED'] = True

    for model in apps.get_models():
        model_name = model.__name__.lower()
        app_label = get_metadata(model, 'app_label')
        add_shortcut = get_metadata(model, 'add_shortcut')
        list_shortcut = get_metadata(model, 'list_shortcut')
        list_diplay = get_metadata(model, 'list_display')
        verbose_name = get_metadata(model, 'verbose_name')
        verbose_name_plural = get_metadata(model, 'verbose_name_plural')
        menu = get_metadata(model, 'menu')
        list_menu = get_metadata(model, 'list_menu')
        dashboard = get_metadata(model, 'dashboard')
        expose = get_metadata(model, 'expose')
        role_signup = get_metadata(model, 'role_signup', False)

        field_names = []
        for field in get_metadata(model, 'get_fields'):
            field_names.append(field.name)
            if hasattr(field, 'composition') and field.composition:
                CACHE['COMPOSITION_FIELDS'][model] = field.name
                if field.remote_field.model not in CACHE['COMPOSITION_RELATIONS']:
                    CACHE['COMPOSITION_RELATIONS'][field.remote_field.model] = []
                if model not in CACHE['COMPOSITION_RELATIONS'][field.remote_field.model]:
                    CACHE['COMPOSITION_RELATIONS'][field.remote_field.model].append(model)

        if model not in CACHE['SUBSETS']:
            CACHE['SUBSETS'][model] = []
        if model not in CACHE['INSTANCE_ACTIONS']:
            CACHE['INSTANCE_ACTIONS'][model] = dict()
        if model not in CACHE['QUERYSET_ACTIONS']:
            CACHE['QUERYSET_ACTIONS'][model] = dict()
        if model not in CACHE['CLASS_ACTIONS']:
            CACHE['CLASS_ACTIONS'][model] = dict()
        if model not in CACHE['FIELDSET_ACTIONS']:
            CACHE['FIELDSET_ACTIONS'][model] = dict()

        if role_signup:
            CACHE['SIGNUP_MODEL'] = model

        # indexing organization model
        if hasattr(model, 'organization_ptr_id'):
            CACHE['ORGANIZATION_MODEL'] = model

        # indexing unit model
        if hasattr(model, 'unit_ptr_id'):
            CACHE['UNIT_MODEL'] = model

        if expose:
            CACHE['API_MODELS'].append(model)

        # indexing shortcuts
        if add_shortcut:
            CACHE['ICON_PANEL_MODELS'].append((model, add_shortcut))
        if list_shortcut:
            CACHE['CARD_PANEL_MODELS'].append((model, list_shortcut))
        if dashboard:
            CACHE['LIST_DASHBOARD'].append(model)

        # indexing the views generated from model classes
        url = '/list/{}/{}/'.format(app_label, model_name)
        icon = None
        if menu and model not in CACHE['COMPOSITION_FIELDS']:
            menu_groups = ()
            if list_menu and list_menu is not True:
                if type(list_menu) != tuple:
                    list_menu = list_menu,
                menu_groups = list_menu
            if type(menu) == tuple:
                description, icon = menu
            else:
                description, icon = menu, get_metadata(model, 'icon')
            permission = '{}.list_{}'.format(app_label, model_name)
            item = dict(
                url=url, can_view=permission, menu=description, icon=icon, add_shortcut=False, groups=menu_groups
            )
            CACHE['VIEWS'].append(item)

        # indexing the @subset and @meta methods defined in the manager classes

        for attr_name in dir(model.objects.get_queryset()):
            attr = getattr(model.objects.get_queryset(), attr_name)
            if hasattr(attr, '_metadata'):
                metadata_type = get_metadata(attr, 'type')
                if metadata_type == 'subset':
                    subset_title = get_metadata(attr, 'verbose_name')
                    subset_name = get_metadata(attr, 'name')
                    subset_help_text = get_metadata(attr, 'help_text')
                    subset_alert = get_metadata(attr, 'alert')
                    subset_notify = get_metadata(attr, 'notify')
                    subset_can_view = get_metadata(attr, 'can_view')
                    subset_order = get_metadata(attr, 'order')
                    subset_menu = get_metadata(attr, 'menu')
                    subset_template = get_metadata(attr, 'template')
                    subset_expose = get_metadata(attr, 'expose')
                    subset_dashboard = get_metadata(attr, 'dashboard')
                    subset_list_display = get_metadata(attr, 'list_display')
                    subset_list_filter = get_metadata(attr, 'list_filter')
                    subset_search_fields = get_metadata(attr, 'search_fields')
                    subset_workflow = get_metadata(attr, 'usecase')
                    subset_url = '{}{}/'.format(url, attr.__func__.__name__)

                    item = dict(
                        verbose_name=subset_title, name=attr_name, function=attr, url=subset_url, can_view=subset_can_view,
                        menu=subset_menu, icon=icon, alert=subset_alert, notify=subset_notify,
                        order=subset_order, help_text=subset_help_text, list_display=subset_list_display,
                        list_filter=subset_list_filter, search_fields=subset_search_fields, expose=subset_expose,
                        template=subset_template
                    )
                    CACHE['SUBSETS'][model].append(item)

                    if subset_dashboard:
                        widget = dict(
                            verbose_name=subset_title, model=model, function=attr_name, can_view=subset_can_view,
                            dashboard=subset_dashboard, formatter=None, link=True, list_display=subset_list_display,
                            list_filter=subset_list_filter, search_fields=subset_search_fields, template=subset_template
                        )
                        CACHE['SUBSET_WIDGETS'].append(widget)

                    if subset_workflow:
                        role = subset_can_view and subset_can_view[0] or 'Superusuário'
                        if attr_name == 'all':
                            activity_description = 'Listar {}'.format(verbose_name_plural,)
                        else:
                            activity_description = 'Listar {}: {}'.format(verbose_name_plural, subset_title)
                        CACHE['WORKFLOWS'][subset_workflow] = dict(activity=activity_description, role=role, model=None)

                # @meta
                else:
                    widget_verbose_name = get_metadata(attr, 'verbose_name')
                    widget_can_view = get_metadata(attr, 'can_view')
                    widget_dashboard = get_metadata(attr, 'dashboard')
                    widget_formatter = get_metadata(attr, 'formatter')
                    widget_shortcut = get_metadata(attr, 'shortcut')
                    widget_icon = get_metadata(attr, 'icon')
                    widget_url = '{}{}/'.format(url, attr.__func__.__name__)
                    widget = dict(
                        verbose_name=widget_verbose_name, model=model, function=attr_name, can_view=widget_can_view,
                        dashboard=widget_dashboard, formatter=widget_formatter, link=False, icon=widget_icon,
                        shortcut=widget_shortcut, url=widget_url
                    )
                    CACHE['SUBSET_WIDGETS'].append(widget)
                    if model not in CACHE['MANAGER_METHODS']:
                        CACHE['MANAGER_METHODS'][model] = list()
                        CACHE['MANAGER_METHODS'][model].append(widget)

        # indexing the actions refered in fieldsets
        if hasattr(model, 'fieldsets'):
            for title, info in model.fieldsets:
                if title not in CACHE['FIELDSET_ACTIONS'][model]:
                    CACHE['FIELDSET_ACTIONS'][model][title] = []
                for action_name in info.get('actions', []):
                    CACHE['FIELDSET_ACTIONS'][model][title].append(action_name)
        else:
            CACHE['SIMPLE_MODELS'].append(model)

        # indexing the actions defined in models
        for attr_name in dir(model):
            if attr_name[0] != '_' and attr_name not in field_names:
                func = getattr(model, attr_name)

                if hasattr(func, '_action'):
                    action = getattr(func, '_action')
                    action_group = action['group']

                    action_can_execute = get_can_execute(action)
                    action_verbose_name = action['verbose_name']
                    action_workflow = action['usecase']
                    action_menu = action['menu']
                    view_name = action['view_name']
                    if action_group not in CACHE['INSTANCE_ACTIONS'][model]:
                        CACHE['INSTANCE_ACTIONS'][model][action_group] = dict()
                    CACHE['INSTANCE_ACTIONS'][model][action_group][view_name] = action
                    if action_workflow:
                        role = action_can_execute and action_can_execute[0] or 'Superusuário'
                        CACHE['WORKFLOWS'][action_workflow] = dict(activity=action_verbose_name, role=role, model=verbose_name)
                    if action_menu:
                        url = '/action/{}/{}/{}/'.format(
                            get_metadata(model, 'app_label'), model.__name__.lower(), attr_name
                        )
                        action_view = dict(
                            verbose_name=action_verbose_name, function=None, url=url, can_view=action_can_execute, menu=action_menu,
                            icon=None, style='ajax', add_shortcut=False, doc=func.__doc__, usecase=None
                        )
                        CACHE['VIEWS'].append(action_view)

                if hasattr(func, '_metadata'):
                    widget_verbose_name = get_metadata(func, 'verbose_name')
                    widget_can_view = get_metadata(func, 'can_view')
                    widget_dashboard = get_metadata(func, 'dashboard')
                    widget_formatter = get_metadata(func, 'formatter')
                    widget_icon = get_metadata(func, 'icon')
                    widget = dict(
                        verbose_name=widget_verbose_name, model=model, function=attr_name, can_view=widget_can_view,
                        dashboard=widget_dashboard, formatter=widget_formatter, link=False, icon=widget_icon
                    )
                    if model not in CACHE['MODEL_WIDGETS']:
                        CACHE['MODEL_WIDGETS'][model] = []
                    CACHE['MODEL_WIDGETS'][model].append(widget)
                    if model not in CACHE['INSTANCE_METHODS']:
                        CACHE['INSTANCE_METHODS'][model] = list()
                        CACHE['INSTANCE_METHODS'][model].append(widget)

        # indexing the actions related to relations whose model has the add_inline meta-attribute
        inlines = []
        if hasattr(model, 'fieldsets'):
            for fieldset in model.fieldsets:
                if 'relations' in fieldset[1]:
                    for item in fieldset[1]['relations']:
                        if ':' in item:
                            # 'relation_name:all[action_a,action_b],subset[action_c]'
                            relation_name = item.split(':')[0]
                        elif '[' in item:
                            # 'relation_name[action_a,action_b]'
                            relation_name = item.split('[')[0]
                        else:
                            # 'relation_name'
                            relation_name = item

        # indexing the actions defined in managers
        qs_manager_class = type(model.objects.get_queryset())
        for attr_name in dir(qs_manager_class):
            if not attr_name[0] == '_':
                attr = getattr(qs_manager_class, attr_name)
                if hasattr(attr, '_action'):
                    action = getattr(attr, '_action')
                    action_verbose_name = action['verbose_name']
                    action_can_execute = get_can_execute(action)
                    action_group = action['group']
                    action_name = action['view_name']
                    action_subsets = action['subsets']
                    action_workflow = action['usecase']
                    is_class_method = isinstance(qs_manager_class.__dict__[attr_name], classmethod)
                    if not action_subsets:
                        action['inline'] = True
                    if is_class_method:
                        if action_group not in CACHE['CLASS_ACTIONS'][model]:
                            CACHE['CLASS_ACTIONS'][model][action_group] = dict()
                        CACHE['CLASS_ACTIONS'][model][action_group][action_name] = action
                    else:
                        if action_group not in CACHE['QUERYSET_ACTIONS'][model]:
                            CACHE['QUERYSET_ACTIONS'][model][action_group] = dict()
                        CACHE['QUERYSET_ACTIONS'][model][action_group][action_name] = action

                    if action_workflow:
                        role = action_can_execute and action_can_execute[0] or 'Superusuário'
                        CACHE['WORKFLOWS'][action_workflow] = dict(activity=action_verbose_name, role=role, model=verbose_name)

    # indexing the formatters
    for app_label in settings.INSTALLED_APPS:
        try:
            module_name = '{}.formatters'.format(app_label)
            module = __import__(module_name, fromlist=list(map(str, app_label.split('.'))))
            for attr_name in dir(module):
                module_attr = getattr(module, attr_name)
                if hasattr(module_attr, '_formatter'):
                    CACHE['FORMATTERS'][getattr(module_attr, '_formatter') or attr_name] = module_attr
        except ImportError as e:
            pass

    from djangoplus.ui.components import Component
    for cls in Component.subclasses():
        formatter_name = cls.formatter_name or cls.__name__.lower()
        if formatter_name not in CACHE['FORMATTERS']:
            CACHE['FORMATTERS'][formatter_name] = cls

    # indexing the actions, views and widgets in views module
    for app_label in settings.INSTALLED_APPS:
            try:
                module = __import__('{}.views'.format(app_label), fromlist=list(map(str, app_label.split('.'))))
                for attr_name in dir(module):
                    func = getattr(module, attr_name)
                    # indexing the actions defined in the views
                    if hasattr(func, '_action'):
                        action = getattr(func, '_action')
                        action_group = action['group']
                        action_model = action['model']
                        action_function = action['function']
                        action_name = action['view_name']
                        action_verbose_name = action['verbose_name']
                        action_workflow = action['usecase']
                        action_can_execute = get_can_execute(action)
                        action_subsets = action['subsets']
                        action_menu = action['menu']

                        if action_workflow:
                            role = action_can_execute and action_can_execute[0] or 'Superusuário'
                            action_model_verbose_name = get_metadata(action_model, 'verbose_name')
                            CACHE['WORKFLOWS'][action_workflow] = dict(activity=action_verbose_name, role=role, model=action_model_verbose_name)
                        # instance action
                        if count_parameters_names(action_function) > 1:
                            if action_group not in CACHE['INSTANCE_ACTIONS'][action_model]:
                                CACHE['INSTANCE_ACTIONS'][action_model][action_group] = dict()
                            CACHE['INSTANCE_ACTIONS'][action_model][action_group][action_name] = action
                        # class action
                        else:
                            if not action_subsets:
                                action['inline'] = True
                            if action_model not in CACHE['CLASS_VIEW_ACTIONS']:
                                CACHE['CLASS_VIEW_ACTIONS'][action_model] = dict()
                            if action_group not in CACHE['CLASS_VIEW_ACTIONS'][action_model]:
                                CACHE['CLASS_VIEW_ACTIONS'][action_model][action_group] = dict()
                            CACHE['CLASS_VIEW_ACTIONS'][action_model][action_group][action_name] = action
                    # indexing the views
                    elif hasattr(func, '_view'):
                        action = getattr(func, '_view')
                        CACHE['VIEWS'].append(action)
                        view_title = action['verbose_name']
                        view_workflow = action['usecase']
                        view_can_view = action['can_view']
                        if view_workflow:
                            role = view_can_view and view_can_view[0] or 'Superusuário'
                            CACHE['WORKFLOWS'][view_workflow] = dict(activity=view_title, role=role, model=None)
                    # indexing the widgets
                    elif hasattr(func, '_widget'):
                        CACHE['WIDGETS'].append(getattr(func, '_widget'))
            except ImportError as e:
                pass

    for model in apps.get_models():
        app_label = get_metadata(model, 'app_label')
        verbose_name = get_metadata(model, 'verbose_name')
        role_username = get_metadata(model, 'role_username')
        role_signup = get_metadata(model, 'role_signup')
        add_label = get_metadata(model, 'add_label', None)
        workflow = get_metadata(model, 'usecase', 0)
        diagram_classes = get_metadata(model, 'class_diagram', None)

        # indexing role models
        if role_username:
            CACHE['ROLE_MODELS'][model] = dict(
                username_field=role_username, scope=get_scope(
                    model, CACHE['ORGANIZATION_MODEL'], CACHE['UNIT_MODEL']), name=verbose_name
            )
        for subclass in model.__subclasses__():
            subclass_role_username = get_metadata(subclass, 'role_username')
            if subclass_role_username:
                subclass_verbose_name = get_metadata(subclass, 'verbose_name')
                CACHE['ROLE_MODELS'][subclass] = dict(username_field=subclass_role_username, scope=get_scope(
                    subclass, CACHE['ORGANIZATION_MODEL'], CACHE['UNIT_MODEL']), name=subclass_verbose_name)
                if model not in CACHE['ABSTRACT_ROLE_MODELS']:
                    CACHE['ABSTRACT_ROLE_MODELS'][model] = []
                    CACHE['ABSTRACT_ROLE_MODEL_NAMES'][verbose_name] = []
                CACHE['ABSTRACT_ROLE_MODELS'][model].append(subclass)
                CACHE['ABSTRACT_ROLE_MODEL_NAMES'][verbose_name].append(subclass_verbose_name)

        permission_by_scope = dict()
        for scope in ('role', 'unit', 'organization'):
            for permission_name in ('edit', 'add', 'delete', 'view', 'list'):
                permission_key = '{}_by_{}'.format(permission_name, scope)
                for group_name in get_metadata(model, 'can_{}'.format(permission_key), (), iterable=True):
                    if permission_name == 'list':
                        permission_key = 'view_by_{}'.format(scope)
                    if permission_key not in permission_by_scope:
                        permission_by_scope[permission_key] = []
                    if group_name in CACHE['ABSTRACT_ROLE_MODEL_NAMES']:
                        for concrete_group_name in CACHE['ABSTRACT_ROLE_MODEL_NAMES'][group_name]:
                            permission_by_scope[permission_key].append(concrete_group_name)
                    else:
                        permission_by_scope[permission_key].append(group_name)
            for group_name in get_metadata(model, 'can_admin_by_{}'.format(scope), (), iterable=True):
                for permission_name in ('edit', 'add', 'delete', 'view', 'list'):
                    if permission_name == 'list':
                        permission_key = 'view_by_{}'.format(scope)
                    else:
                        permission_key = '{}_by_{}'.format(permission_name, scope)
                    if permission_key not in permission_by_scope:
                        permission_by_scope[permission_key] = []
                    if group_name not in permission_by_scope[permission_key]:
                        if group_name in CACHE['ABSTRACT_ROLE_MODEL_NAMES']:
                            for concrete_group_name in CACHE['ABSTRACT_ROLE_MODEL_NAMES'][group_name]:
                                permission_by_scope[permission_key].append(concrete_group_name)
                        else:
                            permission_by_scope[permission_key].append(group_name)

        for permission_name in ('edit', 'add', 'delete', 'view', 'list'):
            permission_key = permission_name
            for group_name in get_metadata(model, 'can_{}'.format(permission_name), (), iterable=True):
                if permission_name == 'list':
                    permission_key = 'view'
                if permission_key not in permission_by_scope:
                    permission_by_scope[permission_key] = []
                if group_name not in permission_by_scope[permission_key]:
                    permission_by_scope[permission_key].append(group_name)
        for group_name in get_metadata(model, 'can_admin', (), iterable=True):
            for permission_name in ('edit', 'add', 'delete', 'view', 'list'):
                permission_key = permission_name
                if permission_name == 'list':
                    permission_key = 'view'
                if permission_key not in permission_by_scope:
                    permission_by_scope[permission_key] = []
                if group_name not in permission_by_scope[permission_key]:
                    permission_by_scope[permission_key].append(group_name)

        for actions_dict in (CACHE['INSTANCE_ACTIONS'], CACHE['QUERYSET_ACTIONS']):
            for category in actions_dict[model]:
                for key in list(actions_dict[model][category].keys()):
                    name = actions_dict[model][category][key]['verbose_name']
                    view_name = actions_dict[model][category][key]['view_name']
                    can_execute = []
                    for scope in ('', 'role', 'unit', 'organization'):
                        scope = scope and '_by_{}'.format(scope) or scope
                        for group_name in actions_dict[model][category][key].get('can_execute{}'.format(scope)) or ():
                            permission_key = '{}{}'.format(view_name, scope)
                            if permission_key not in permission_by_scope:
                                permission_by_scope[permission_key] = []
                            permission_by_scope[permission_key].append(group_name)

        if permission_by_scope:
            CACHE['PERMISSIONS_BY_SCOPE'][model] = permission_by_scope

        if workflow:
            role = permission_by_scope.get('add_by_role') and permission_by_scope.get('add_by_role')[0] or None
            if not role:
                role = permission_by_scope.get('add_by_unit') and permission_by_scope.get('add_by_unit')[0] or None
            if not role:
                role = permission_by_scope.get('add_by_organization') and permission_by_scope.get('add_by_organization')[0] or None
            if not role:
                role = permission_by_scope.get('add') and permission_by_scope.get('add')[0] or None
            if not role or role == verbose_name:
                role = 'Superusuário'

            if model in CACHE['COMPOSITION_FIELDS']:
                related_model = getattr(model, CACHE['COMPOSITION_FIELDS'][model]).field.remote_field.model
                related_verbose_name = get_metadata(related_model, 'verbose_name')
                related_add_label = get_metadata(model, 'add_label')
                if related_add_label:
                    related_add_label = related_add_label.replace(' em ', ' __ ')
                    activity = '{} em {}'.format(related_add_label, related_verbose_name)
                else:
                    verbose_name = verbose_name.replace(' em ', ' __ ')
                    activity = 'Adicionar {} em {}'.format(verbose_name, related_verbose_name)
                CACHE['WORKFLOWS'][workflow] = dict(activity=activity, role=role, model=None)
            else:
                if add_label:
                    activity = add_label
                else:
                    if role_signup:
                        activity = '{} {}'.format('Cadastrar-se como', verbose_name)
                        role = verbose_name
                    else:
                        activity = '{} {}'.format('Cadastrar', verbose_name)
                CACHE['WORKFLOWS'][workflow] = dict(activity=activity, role=role, model=None)

        if diagram_classes is not None:
            CACHE['CLASS_DIAGRAMS'][verbose_name] = [model]
            if type(diagram_classes) == bool and diagram_classes:
                for field in model._meta.get_fields():
                    if field.remote_field and field.remote_field.model:
                        if field.remote_field.model not in CACHE['CLASS_DIAGRAMS'][verbose_name]:
                            CACHE['CLASS_DIAGRAMS'][verbose_name].append(field.remote_field.model)
            else:
                for model_name in diagram_classes:
                    try:
                        extra_model = apps.get_model(app_label, model_name)
                    except LookupError:
                        for extra_model in apps.get_models():
                            if extra_model.__name__.lower() == model_name:
                                break
                    if extra_model not in CACHE['CLASS_DIAGRAMS'][verbose_name]:
                        CACHE['CLASS_DIAGRAMS'][verbose_name].append(extra_model)

    keys = list(CACHE['WORKFLOWS'].keys())
    keys.sort()
    l = []
    for key in keys:
        l.append(CACHE['WORKFLOWS'][key])
    CACHE['WORKFLOWS'] = l

    if settings.DROPBOX_TOKEN and settings.DEBUG:
        def sync_storage():
            from djangoplus.utils.storage.dropbox import DropboxStorage
            DropboxStorage().sync()
        Thread(target=sync_storage).start()




# -*- coding: utf-8 -*-

import json
from django.conf import settings
from djangoplus.cache import CACHE
from django.utils import translation
from django.utils.translation import ugettext as _
from djangoplus.utils.metadata import get_metadata


class Workflow(object):

    def __init__(self):

        translation.activate(settings.LANGUAGE_CODE)

        self.actors = []
        self.tasks = []
        tmp = None
        for task in CACHE['WORKFLOWS']:
            role = task['role']
            activity = task['activity']
            model = task['model']

            if role != tmp:
                tmp = role or _('Superuser')
                action = _('Acessar como')
                self.tasks.append('{} {}'.format(action, tmp))

            if model:
                action = '{}{}{}'.format(activity, _(' in '), model)
            else:
                action = activity
            self.tasks.append(action)


class ClassDiagram(object):

    POSITION_MAP = {
        1: (2.2,), 2: (1.2, 3.2,), 3: (3.2, 1.1, 1.3,), 4: (2.2, 3.2, 1.1, 1.3,), 5: (2.2, 1.1, 1.3, 3.1, 3.3,),
        6: (1.2, 3.2, 1.1, 3.1, 1.3, 3.3,), 7: (2.2, 1.2, 3.2, 1.1, 1.3, 3.1, 3.3,),
        8: (1.2, 3.2, 1.1, 1.3, 2.1, 2.3, 3.1, 3.3,), 9: (1.1, 1.2, 1.3, 2.1, 2.2, 2.3, 3.1, 3.2, 3.3,)
    }

    def __init__(self, class_diagram_name, models):

        self.name = class_diagram_name
        self.classes = []
        self.compositions = []
        self.agregations = []

        classes = dict()
        n = len(CACHE['CLASS_DIAGRAMS'][class_diagram_name])
        associations_count = {}
        for model in CACHE['CLASS_DIAGRAMS'][class_diagram_name]:
            associations_count[model] = 0

        for model in CACHE['CLASS_DIAGRAMS'][class_diagram_name]:
            verbose_name = get_metadata(model, 'verbose_name')
            related_objects = get_metadata(model, 'related_objects')
            classes[model] = dict(name=verbose_name, position='1.1')
            for related_object in related_objects:
                related_verbose_name = get_metadata(related_object.related_model, 'verbose_name')
                if related_object.related_model in models:
                    if hasattr(related_object.field, 'composition') and related_object.field.composition:
                        self.compositions.append(
                            [related_verbose_name, verbose_name, related_object.remote_field.name])
                        associations_count[model] += 1
                        associations_count[related_object.related_model] += 1
                    else:
                        first_condition = model not in CACHE['ROLE_MODELS'] or class_diagram_name == related_verbose_name
                        second_condition = class_diagram_name == verbose_name and related_object.related_model not in \
                            CACHE['ROLE_MODELS']
                        if first_condition or second_condition:
                            self.agregations.append(
                                [verbose_name, related_verbose_name, related_object.field.name]
                            )
                            associations_count[related_object.related_model] += 1
                            associations_count[model] += 1
        sorted_associations_count = sorted(associations_count, key=associations_count.get, reverse=True)
        for i, model in enumerate(sorted_associations_count):
            cls = classes[model]
            cls['position'] = ClassDiagram.POSITION_MAP[n][i]
            self.classes.append(cls)

    def as_dict(self):
        return dict(name=self.name, classes=self.classes, compositions=self.compositions, agregations=self.agregations)

    def as_json(self):
        return json.dumps(self.as_dict())



# -*- coding: utf-8 -*-
import os
from django.conf import settings
from subprocess import Popen, DEVNULL
from django.core.management.base import BaseCommand

from djangoplus.tools.video import VideoUploader, VideoRecorder


class Command(BaseCommand):

    def add_arguments(self, parser):
        super(Command, self).add_arguments(parser)
        parser.add_argument('video', default=None, nargs='?')
        parser.add_argument('--delete', action='store_true', dest='delete', default=False, help='Deletes the video')
        parser.add_argument('--upload', action='store_true', dest='upload', default=False,
                            help='Uploads the video if it has not been uploaded yet')
        parser.add_argument('--force-upload', action='store_true', dest='force_upload', default=False,
                            help='Uploads the video even it has already been uploaded')
        parser.add_argument('--clear', action='store_true', dest='clear', default=False,
                            help='Deletes all uploaded videos')

    def handle(self, *args, **options):
        youtube = VideoUploader()
        video = options.get('video')
        delete = options.get('delete')
        upload = options.get('upload')
        force_upload = options.get('force_upload')
        clear = options.get('clear')
        if video:
            if upload or force_upload:
                video_path = '{}/{}'.format(settings.BASE_DIR, video)
                if os.path.exists(video_path):
                    title = VideoRecorder.metadata(video_path).get('title', video)
                    youtube.upload_video(video_path, title, force=force_upload)
                else:
                    print('Video {} does not exists.'.format(video_path))
            elif delete:
                if video.endswith('.mkv'):
                    video_path = '{}/{}'.format(settings.BASE_DIR, video)
                    if os.path.exists(video_path):
                        os.unlink(video_path)
                        print('The video {} deleted from file system.'.format(video_path))
                    else:
                        print('The video {} does not exists.'.format(video_path))
                else:
                    youtube.delete_video(video)
            else:
                vlc = None
                if os.path.exists('/Applications/VLC.app/Contents/MacOS/VLC'):
                    vlc = '/Applications/VLC.app/Contents/MacOS/VLC'
                elif os.path.exists('/usr/bin/vlc'):
                    vlc = '/usr/bin/vlc'
                if vlc:
                    cmd = '{} --play-and-exit --fullscreen {}'.format(vlc, video)
                    Popen(cmd.split(), stdin=DEVNULL, stdout=DEVNULL, stderr=DEVNULL)
                else:
                    print('VLC is not installed!')
        else:
            directory = '{}/videos'.format(settings.BASE_DIR)

            local_videos = []
            if os.path.exists(directory):
                for file_name in os.listdir(directory):
                    if '.mkv' in file_name:
                        url = 'videos/{}'.format(file_name)
                        video_path = os.path.join(directory, file_name)
                        title = VideoRecorder.metadata(video_path).get('title', file_name)
                        local_video = dict(file_name=file_name, url=url, title=title, video_path=video_path)
                        local_videos.append(local_video)

            if upload or force_upload:
                for local_video in local_videos:
                    youtube.upload_video(local_video['video_path'], local_video['title'], force=force_upload)
            elif clear:
                for uploaded_video in youtube.list_videos():
                    youtube.delete_video(uploaded_video['id'])
            else:
                print('LOCAL VIDEOS')
                if local_videos:
                    for local_video in local_videos:
                        print('{}\t{}\t{}'.format(local_video['file_name'], local_video['url'], local_video['title']))
                else:
                    print('No local video was found!')

                print('\nUPLOADED VIDEOS')
                uploaded_videos = youtube.list_videos()
                if uploaded_videos:
                    for uploaded_video in uploaded_videos:
                        url = 'https://www.youtube.com/watch?v={}'.format(uploaded_video['id'])
                        print('{}\t{}\t{}'.format(uploaded_video['id'], url, uploaded_video['snippet']['title']))
                else:
                    print('No uploaded video was found!')



# -*- coding: utf-8 -*-

import copy
from collections import OrderedDict
from djangoplus.ui.components import forms
from django.db.models.aggregates import Sum
from djangoplus.utils.tabulardata import tolist
from djangoplus.utils.formatter import normalyze
from djangoplus.ui.components.forms import factory
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext as _
from djangoplus.utils import permissions, should_add_action, get_role_values_for_condition
from djangoplus.ui.components.navigation.breadcrumbs import httprr
from djangoplus.ui.components import Component, ComponentHasResponseException
from djangoplus.utils.http import CsvResponse, XlsResponse, ReportResponse, return_response
from djangoplus.ui.components.navigation.dropdown import ModelDropDown, GroupDropDown
from djangoplus.utils.metadata import get_metadata, get_field, get_fiendly_name, should_filter_or_display, getattr2, \
    get_parameters_names, count_parameters_names


class Paginator(Component):
    def __init__(self, request, qs, title=None, list_display=None, list_filter=None, search_fields=None,
                 list_per_page=25, list_subsets=None, exclude=None, relation=None, readonly=False, is_list_view=False,
                 help_text=None, url=None, template=None, uid=None):

        super(Paginator, self).__init__(is_list_view and '_' or abs(hash(title)), request)
        if relation:
            qs = qs.model.objects.filter(pk__in=qs.values_list('pk', flat=True))
        else:
            qs = qs.all()
        self.qs = qs
        self.title = title
        self.list_display = list_display
        self.list_filter = list_filter
        self.search_fields = search_fields
        self.list_per_page = list_per_page
        self.subsets = None
        self.exclude = exclude
        self.relation = relation
        self.readonly = readonly
        self.is_list_view = is_list_view
        self.icon = get_metadata(qs.model, 'icon', None)
        self.list_total = get_metadata(qs.model, 'list_total', None)
        self.ordering = get_metadata(qs.model, 'ordering', None)
        self.template = template or get_metadata(self.qs.model, 'list_template', 'datagrid.html')
        self.help_text = help_text
        self.url = url
        self.display_checkboxes = False
        self.action_names = None

        self.filters = []
        self.pagination = ''

        self.original_qs = qs
        self.count = None

        # tabs
        self.tabs = []
        self.current_tab = self._get_from_request('tab', None)
        if list_subsets is not None:
            self.subsets = OrderedDict()
            for subset_name in list_subsets:
                self.subsets[subset_name] = {}
        self._load_tabs()

        # list display
        self._configure_list_display()

        # column names
        self.column_names = []
        self._configure_column_names()

        # drop down
        self.class_dropdown = GroupDropDown(request, style='class-action')
        self.queryset_dropdown = GroupDropDown(request, style='queryset-action disabled')
        self.drop_down = ModelDropDown(request, qs.model)

        if hasattr(self.qs, 'permission_map'):
            self.permission_map = self.qs.permission_map
        self.qs = self._filter_queryset(self.qs)

    def get_current_tab_name(self):
        if len(self.tabs) > 1:
            if self.current_tab and self.current_tab != 'all':
                return self.current_tab
        if len(self.tabs) > 0:
            return self.tabs[0][0]
        return ''
    
    def get_tab(self):
        return self._get_from_request('tab', '')
    
    def get_search_fields(self):
        if self.search_fields is not None:
            return self.search_fields
        return get_metadata(self.qs.model, 'search_fields', [])
    
    def get_list_filter(self):
        if self.list_filter is None:
            self.list_filter = copy.deepcopy(get_metadata(self.qs.model, 'list_filter', None))
            if self.list_filter is None:
                self.list_filter = []
                for field in get_metadata(self.qs.model, 'fields'):
                    if field.remote_field and field.remote_field.model:
                        if not field.name.endswith('_ptr') and field.name != self.rel:
                            self.list_filter.append(field.name)
                    elif hasattr(field, 'choices') and field.choices:
                        self.list_filter.append(field.name)
                    elif type(field).__name__ in ['BooleanField', 'NullBooleanField', 'DateField', 'DateTimeField']:
                        self.list_filter.append(field.name)
        return self.list_filter

    def get_filter_form(self):
        form = forms.Form(self.request)
        form.title = 'Filtros'
        form.icon = 'fa-filter'
        form.partial = True
        self.filters = []
        for list_filter in self.get_list_filter():
            if type(list_filter) in (tuple, list):
                field_names = list_filter
            else:
                field_names = list_filter,
            for field_name in field_names:
                if self.relation and field_name == self.relation.hidden_field_name:
                    continue
                field = get_field(self.qs.model, field_name)
                form_field_name = '{}{}'.format(field_name, self.id)
                if hasattr(field, 'auto_now'):
                    initial = (self._get_from_request(field_name, None, '_0'), self._get_from_request(field_name, None, '_1'))
                    form.fields[form_field_name] = forms.DateFilterField(label=normalyze(field.verbose_name), initial=initial, required=False)
                else:
                    initial = self._get_from_request(field_name)
                    if type(field).__name__ in ['BooleanField', 'NullBooleanField']:
                        form.fields[form_field_name] = forms.ChoiceField(
                            choices=[['', ''], ['sim', 'Sim'], ['nao', 'Não'], ], label=normalyze(field.verbose_name), initial=initial, required=False)
                    elif hasattr(field, 'choices') and field.choices:
                        form.fields[form_field_name] = forms.ChoiceField(choices=[['', '']] + field.choices, label=normalyze(field.verbose_name), initial=initial, required=False)
                    else:
                        if hasattr(field.remote_field.model, 'unit_ptr_id') and self.request.user.role_set.filter(scope__unit__isnull=False).values_list('scope__unit', flat=True).count() == 1:
                            continue
                        if hasattr(field.remote_field.model, 'organization_ptr_id') and self.request.user.role_set.filter(scope__organization__isnull=False).values_list('scope__organization', flat=True).count() == 1:
                            continue
                        if field.remote_field and not should_filter_or_display(self.request, self.qs.model, field.remote_field.model):
                            continue
                        if initial:
                            if self.original_qs.query.can_filter():
                                pks = self.original_qs.order_by(field_name).values_list(field_name, flat=True).distinct()
                            else:
                                pks = self.original_qs.model.objects.all().order_by(field_name).values_list(field_name, flat=True).distinct()
                            qs = field.remote_field.model.objects.get_queryset().filter(pk__in=pks)
                            qs = qs | field.remote_field.model.objects.get_queryset().filter(pk=initial)
                        else:
                            if self.qs.query.can_filter():
                                pks = self.qs.order_by(field_name).values_list(field_name, flat=True).distinct()
                            else:
                                pks = self.qs.model.objects.all().order_by(field_name).values_list(field_name, flat=True).distinct()
                            qs = field.remote_field.model.objects.get_queryset().filter(pk__in=pks)
                        empty_label = ''

                        form.fields[form_field_name] = forms.ModelChoiceField(qs, label=normalyze(field.verbose_name), initial=initial, empty_label=empty_label, required=False, lazy=False, ignore_lookup=True, minimum_input_length=0)
                    form.fields[form_field_name].widget.attrs['data-placeholder'] = field.verbose_name
                    if initial:
                        label = form.fields[form_field_name].label
                        value = form.fields[form_field_name].clean(initial)
                        if type(form.fields[form_field_name]) == forms.ChoiceField:
                            for x, y in form.fields[form_field_name].choices:
                                if str(x) == str(value):
                                    value = y
                                    break
                        self.filters.append((label, value))
        return form

    def get_selected_ids(self):
        ids = self.request.GET.get('ids', None)
        if ids:
            ids = ids.split(',')
            ids = 0 not in ids and ids or None
        return ids

    def add_action(self, label, url, css, icon=None, category=None):
        self.class_dropdown.add_action(label, url, 'class-action {}'.format(css), icon, category)

    def add_queryset_action(self, label, url, css, icon=None, category=None, subset=None):
        if not subset or self.get_tab() == subset:
            if not self.mobile:
                self.queryset_dropdown.add_action(label, url, 'queryset-action disabled {}'.format(css), icon, category)

    def load_actions(self, action_names=None):

        from djangoplus.cache import CACHE
        export_url = self.request.get_full_path()
        list_csv = get_metadata(self.qs.model, 'list_csv')
        list_xls = get_metadata(self.qs.model, 'list_xls')
        log = get_metadata(self.qs.model, 'log')
        app_label = get_metadata(self.qs.model, 'app_label')
        list_pdf = get_metadata(self.qs.model, 'list_pdf')
        self.action_names = action_names or []

        subset = self.subsets and list(self.subsets.keys())[0] or None
        subsetp = self.request.GET.get('tab{}'.format(self.request.GET.get('tid')))
        subset_name = subsetp or subset

        # class actions defined in the manager
        if self.qs.model in CACHE['CLASS_ACTIONS']:
            for group in CACHE['CLASS_ACTIONS'][self.qs.model]:
                for view_name in CACHE['CLASS_ACTIONS'][self.qs.model][group]:
                    _action = CACHE['CLASS_ACTIONS'][self.qs.model][group][view_name]
                    action_verbose_name = _action['verbose_name']
                    action_inline = _action['inline']
                    action_subsets = _action['subsets']
                    action_can_execute = _action['can_execute']
                    action_category = _action['group']
                    action_expose = _action['expose']
                    action_style = _action['style'] or 'popup'
                    if 'popup' not in action_style:
                        action_style = '{} popup'.format(action_style)
                    if self.is_list_view or view_name in self.action_names:
                        add_action = should_add_action(action_inline, action_subsets, subset_name)
                        add_action = add_action and (True in action_expose or 'web' in action_expose)
                        if add_action and permissions.check_group_or_permission(self.request, action_can_execute):
                            func = getattr(self.qs, view_name)
                            ignore_pdf = False
                            if count_parameters_names(func) > 0:
                                action_style = action_style.replace('pdf', '')
                                ignore_pdf = True
                            char = '?' in self.request.get_full_path() and '&' or '?'
                            url = '{}{}{}'.format(self.request.get_full_path(), char, '{}='.format(view_name))
                            self.add_action(action_verbose_name, url, action_style, None, action_category)
                            if view_name in self.request.GET:
                                self._proccess_request(func, _action, ignore_pdf)

        # subset actions defined in the manager
        if self.qs.model in CACHE['QUERYSET_ACTIONS']:
            for group in CACHE['QUERYSET_ACTIONS'][self.qs.model]:
                for view_name in CACHE['QUERYSET_ACTIONS'][self.qs.model][group]:
                    _action = CACHE['QUERYSET_ACTIONS'][self.qs.model][group][view_name]
                    action_verbose_name = _action['verbose_name']
                    action_can_execute = _action['can_execute']
                    action_inline = _action['inline']
                    action_icon = _action['icon']
                    action_subsets = _action['subsets']
                    action_condition = _action['condition']
                    action_source = _action['source']
                    action_category = _action['group']
                    action_expose = _action['expose']
                    action_style = _action['style'] or 'popup'
                    if 'popup' not in action_style:
                        action_style = '{} popup'.format(action_style)
                    if view_name in self.action_names or (self.is_list_view and action_inline) or (self.current_tab in action_subsets):
                        add_action = should_add_action(action_inline, action_subsets, subset_name)
                        add_action = add_action and (True in action_expose or 'web' in action_expose)
                        if add_action and permissions.check_group_or_permission(self.request, action_can_execute):
                            self.display_checkboxes = self.display_checkboxes or not action_source == 'view'
                            func = getattr(self.qs, view_name)
                            ignore_pdf = False
                            char = '?' in self.request.get_full_path() and '&' or '?'
                            url = '{}{}{}'.format(self.request.get_full_path(), char, '{}='.format(view_name))
                            if count_parameters_names(func) > 0:
                                ignore_pdf = True
                                action_style = action_style.replace('pdf', '')
                            self.add_queryset_action(
                                action_verbose_name, url, action_style, action_icon, action_category, action_condition
                            )

                            if view_name in self.request.GET:
                                ids = self.get_selected_ids()
                                if ids:
                                    qs = self.get_queryset(paginate=False).filter(id__in=ids)
                                else:
                                    break
                                func = getattr(qs, view_name)
                                self._proccess_request(func, _action, ignore_pdf)

        # class actions defined in views module
        if self.qs.model in CACHE['CLASS_VIEW_ACTIONS']:
            for group in CACHE['CLASS_VIEW_ACTIONS'][self.qs.model]:
                for view_name in CACHE['CLASS_VIEW_ACTIONS'][self.qs.model][group]:
                    _action = CACHE['CLASS_VIEW_ACTIONS'][self.qs.model][group][view_name]
                    action_verbose_name = _action['verbose_name']
                    action_inline = _action['inline']
                    action_subsets = _action['subsets']
                    action_can_execute = _action['can_execute']
                    action_category = _action['group']
                    action_expose = _action['expose']
                    action_style = _action['style'] or 'popup'
                    url = '/{}/{}/'.format(get_metadata(self.qs.model, 'app_label'), view_name)
                    action_style = action_style.replace('popup', '')
                    if self.is_list_view or view_name in self.action_names:
                        add_action = should_add_action(action_inline, action_subsets, subset_name)
                        add_action = add_action and (True in action_expose or 'web' in action_expose)
                        if add_action and permissions.check_group_or_permission(self.request, action_can_execute):
                            self.add_action(action_verbose_name, url, action_style, None, action_category)

        if self.is_list_view:
            # utility actions
            if list_csv:
                export_url = '?' in export_url and '{}&export=csv'.format(export_url) or '{}?export=csv'.format(export_url)
                self.add_action('Exportar CSV', export_url, 'ajax', 'fa-table')

            if list_xls:
                export_url = '?' in export_url and '{}&export=excel'.format(export_url) or '{}?export=excel'.format(export_url)
                self.add_action('Exportar Excel', export_url, 'ajax', 'fa-file-excel-o')

            if log:
                log_url = '/log/{}/{}/'.format(app_label, self.qs.model.__name__.lower())
                if self.request.user.has_perm('admin.list_log'):
                    self.add_action('Visualizar Log', log_url, 'ajax', 'fa-history')

            if list_pdf:
                pdf_url = '?' in export_url and '{}&export=pdf'.format(export_url) or '{}?export=pdf'.format(export_url)
                self.add_action('Imprimir', pdf_url, 'ajax', 'fa-print')

            # registration action
            subclasses = self.qs.model.__subclasses__()
            if not subclasses and not self.subsets and permissions.has_add_permission(self.request, self.qs.model):
                instance = self.qs.model()
                instance._user = self.request.user
                if not hasattr(instance, 'can_add') or instance.can_add():
                    if self.relation:
                        verbose_name = get_metadata(self.qs.model, 'verbose_name')
                        add_label = get_metadata(self.qs.model, 'add_label', '{} {}'.format(_('Add'), verbose_name))
                    else:
                        add_label = get_metadata(self.qs.model, 'add_label', _('Add'))
                    add_url = '/add/{}/{}/'.format(app_label, self.qs.model.__name__.lower())
                    self.add_action(add_label, add_url, 'ajax', 'fa-plus')

            for subclass in subclasses:
                app = get_metadata(subclass, 'app_label')
                verbose_name = get_metadata(subclass, 'verbose_name')
                cls = subclass.__name__.lower()
                if permissions.has_add_permission(self.request, subclass):
                    self.add_action(verbose_name, '/add/{}/{}/'.format(app, cls), False, 'fa-plus')

    def _proccess_request(self, func, _action, ignore_pdf):

        form = None
        f_return = None
        redirect_url = None

        if count_parameters_names(func) > 0:
            form = factory.get_class_action_form(self.request, self.qs.model, _action, func)
            if form.is_valid():
                params = []
                for param in get_parameters_names(func):
                    if param in form.cleaned_data:
                        params.append(form.cleaned_data[param])
                try:
                    f_return = func(*params)
                    ignore_pdf = False
                    if f_return is None:
                        redirect_url = '..'
                except ValidationError as e:
                    form.add_error(None, str(e.message))
        else:
            f_return = func()
            ignore_pdf = False
            if f_return is None:
                redirect_url = '.'

        if redirect_url:
            self.request.GET._mutable = True
            if 'ids' in self.request.GET:
                del self.request.GET['ids']
            del self.request.GET[_action['view_name']]
            self.request.GET._mutable = False
            raise ComponentHasResponseException(
                httprr(self.request, redirect_url, _action['message'])
            )
        else:
            action_style = _action['style']
            if f_return is None:
                f_return = dict(form=form)
                template_name = 'default.html'
            else:
                template_name = '{}.html'.format(_action['view_name'])
            return_response(
                f_return, self.request, None, action_style, template_name,
                raise_response=True, ignore_pdf=ignore_pdf
            )

    def get_total(self):
        if self.list_total:
            return self.qs.aggregate(sum=Sum(self.list_total)).get('sum')
        else:
            return None

    def can_show_actions(self):
        return permissions.has_view_permission(self.request, self.qs.model)

    def get_queryset(self, paginate=True):
        queryset = self.qs
        self.count = queryset.count()
        if self.ordering:
            if not type (self.ordering) == tuple:
                self.ordering = self.ordering,
            queryset = queryset.order_by(*self.ordering)
        if self.get_list_filter():
            queryset = queryset.distinct()
        if paginate:
            l = []

            list_per_page = self._get_list_per_page()
            page_numer = int(self.count / list_per_page + (((self.count % list_per_page) > 0 or self.count < list_per_page) and 1 or 0))
            current_page = int(self._get_from_request('page', 1))
            start = current_page * list_per_page - list_per_page
            end = start + list_per_page
            queryset = queryset[start:end]
            if page_numer > 1:
                l.append(
                    '<ul class="pagination pagination-xs m-top-none pull-right pagination-split">'
                    '<li class="disabled"><a href="#!"><i class="fa fa-chevron-left"></i></a></li>')
                for i in range(current_page-10, current_page+10):
                    if page_numer >= i > 0 and i:
                        onclick = "$('#page{}').val({});$('#{}').submit();".format(self.id, i, self.id)
                        if i == current_page:
                            css = 'active'
                        else:
                            css = 'waves-effect'
                        l.append('<li class="{}"><a href="javascript:" onclick="{}">{}</a></li>'.format(css, onclick, i))
                l.append('<li class="disabled"><a href="#!"><i class="fa fa-chevron-right"></i></a></li></ul>')
            self.pagination = ''.join(l)

        select_related = get_metadata(queryset.model, 'select_related')
        if select_related:
            queryset = queryset.select_related(*select_related)
        return queryset
    
    def get_q(self):
        return self._get_from_request('q', '')

    def _configure_list_display(self):
        hidden_fields = []
        if not self.list_display:
            self.list_display = list(get_metadata(self.qs.model, 'list_display', []))
        if not self.list_display:
            fields = []
            for field in get_metadata(self.qs.model, 'fields'):
                if not hasattr(field, 'display') or field.display:
                    fields.append(field)
            for field in get_metadata(self.qs.model, 'local_many_to_many'):
                if not hasattr(field, 'display') or field.display:
                    fields.append(field)
            for field in fields[1:6]:
                if not field.name.endswith('_ptr') and not field.name == 'ascii' and not type(field).__name__ == 'TreeIndexField':
                    if not hasattr(field, 'display') or field.display:
                        self.list_display.append(field.name)

        for field_name in self.list_display:
            if '__' in field_name:
                attr = getattr2(self.qs.model, field_name)
            else:
                attr = getattr(self.qs.model, field_name)
            if hasattr(attr, 'field_name'):
                field = getattr(self.qs.model, '_meta').get_field(attr.field_name)
                if hasattr(field, 'display') and not field.display:
                    hidden_fields.append(field_name)

            if self.relation and (field_name == self.relation.hidden_field_name or field_name.startswith('{}__'.format(self.relation.hidden_field_name))):
                hidden_fields.append(field_name)

        if self.exclude:
            for field_name in self.exclude:
                if field_name in self.list_display:
                    hidden_fields.append(field_name)

        for field_name in hidden_fields:
            self.list_display.remove(field_name)

    def _configure_column_names(self):
        for lookup in self.list_display:
            hide_field = False
            attr = getattr(self.qs.model, lookup.split('__')[0])
            if hasattr(attr, 'field') and attr.field.remote_field and attr.field.remote_field.model:
                if hasattr(attr.field.remote_field.model, 'unit_ptr_id') and self.request.user.role_set.filter(scope__unit__isnull=False).values_list('scope__unit', flat=True).count() == 1:
                    continue
                if hasattr(attr.field.remote_field.model, 'organization_ptr_id') and self.request.user.role_set.filter(scope__organization__isnull=False).values_list('scope__organization', flat=True).count() == 1:
                    continue
                if not should_filter_or_display(self.request, self.qs.model, attr.field.remote_field.model):
                    hide_field = True
            if not hide_field:
                self.column_names.append(get_fiendly_name(self.qs.model, lookup, as_tuple=True))

    def _load_tabs(self):
        from djangoplus.cache import CACHE

        subsets = []
        if self.subsets is None:
            subsets = CACHE['SUBSETS'][self.qs.model]
        elif self.subsets:
            for subset_name in self.subsets.keys():
                for subset in CACHE['SUBSETS'][self.qs.model]:
                    if subset['name'] == subset_name:
                        subsets.append(subset)

        tab_qs = None
        active_queryset = None
        create_default_tab = True
        for subset in subsets:
            tab_title = subset['verbose_name']
            tab_function = subset['function']
            tab_can_view = subset['can_view']
            tab_name = subset['name']
            tab_help_text = subset['help_text']
            tab_order = subset['order']
            tab_list_display = subset['list_display']
            tab_list_filter = subset['list_filter']
            tab_search_fields = subset['search_fields']
            tab_active = False
            tab_expose = subset['expose']
            if True not in tab_expose and 'web' not in tab_expose:
                continue
            if permissions.check_group_or_permission(self.request, tab_can_view):
                func = getattr(self.qs, tab_function.__func__.__name__)
                params = get_role_values_for_condition(func, self.request.user)
                tab_qs = func(*params)
                tab_active = self.current_tab == tab_name
                self.tabs.append([tab_name, tab_title, tab_qs, tab_active, tab_order, tab_help_text])
                if (tab_active or len(subsets) == 1) and tab_help_text:
                    self.help_text = tab_help_text
            if tab_name == 'all':
                create_default_tab = False
            if tab_active:
                active_queryset = tab_qs
                self.drop_down = ModelDropDown(self.request, self.qs.model)
                self.list_display = tab_list_display
                self.list_filter = tab_list_filter
                self.search_fields = tab_search_fields

        self.tabs = sorted(self.tabs, key=lambda k: k[4])
        if (self.subsets is None or self.relation) and create_default_tab:
            tab_title = get_metadata(self.qs.model, 'verbose_female') and 'Todas' or 'Todos'
            tab_qs = self.original_qs.all()
            tab_active = self.current_tab == 'all'
            if tab_active:
                active_queryset = tab_qs
            tab_order = 0
            self.tabs.insert(0, ['', tab_title, tab_qs, tab_active, tab_order, None])
        if active_queryset is not None:
            self.qs = active_queryset
        elif self.tabs:
            self.tabs[0][3] = True
            self.qs = self.tabs[0][2]

    def _get_from_request(self, param_name, default='', suffix=''):
        key = '{}{}{}'.format(param_name, self.id, suffix)
        value = self.request.GET.get(str(self.id)) and self.request.GET.get(key) or default
        return value

    def _get_order_by(self):
        return self._get_from_request('order_by', '')

    def _get_page(self):
        return self._get_from_request('page', 1)

    def _get_list_per_page(self):
        return get_metadata(self.qs.model, 'list_per_page', self.list_per_page)

    def _filter_queryset(self, qs):
        distinct = False
        queryset = None
        search_fields = self.get_search_fields()
        q = self.get_q()
        if q:
            for i, search_field in enumerate(search_fields):
                if i == 0:
                    queryset = qs.filter(**{'{}__icontains'.format(search_field): q})
                else:
                    queryset = queryset | qs.filter(**{'{}__icontains'.format(search_field): q})
        else:
            queryset = qs
        for field_name in self.get_list_filter():
            field = get_field(queryset.model, field_name)
            if type(field).__name__ == 'DateField':
                filter_value = self._get_from_request(field_name, None, '_0')
                if filter_value:
                    date, month, year = filter_value.split('/')
                    filter_value = '{}-{}-{}'.format(year, month, date)
                    queryset = queryset.filter(**{'{}__gte'.format(field_name): filter_value})
                filter_value = self._get_from_request(field_name, None, '_1')
                if filter_value:
                    date, month, year = filter_value.split('/')
                    filter_value = '{}-{}-{}'.format(year, month, date)
                    queryset = queryset.filter(**{'{}__lte'.format(field_name): filter_value})
            elif type(field).__name__ in ['BooleanField', 'NullBooleanField']:
                filter_value = self._get_from_request(field_name)
                if filter_value:
                    filter_value = filter_value == 'sim'
                    queryset = queryset.filter(**{field_name: filter_value})
            else:
                filter_value = self._get_from_request(field_name)
                if filter_value:
                    queryset = queryset.filter(**{field_name: filter_value})
                if type(field).__name__ == 'ManyToManyField':
                    distinct = True
        order_by = self._get_from_request('order_by')

        tree_index_field = hasattr(self.qs.model, 'get_tree_index_field') and self.qs.model().get_tree_index_field() or None
        if tree_index_field:
            queryset = queryset.order_by(tree_index_field.name)
        else:
            if order_by:
                queryset = queryset.order_by(order_by.replace('0', ''))
            else:
                order_by = get_metadata(self.qs.model, 'order_by', iterable=True)
                try:
                    queryset = queryset.order_by(*order_by)
                except AssertionError:
                    # if the querycet was sliced
                    pass
        if distinct:
            queryset = queryset.distinct()
        return queryset.all(self.request.user)

    def process_request(self):
        export = self.request.GET.get('export', None)
        if export == 'pdf':
            raise ComponentHasResponseException(self._get_pdf_response())
        if export == 'csv':
            raise ComponentHasResponseException(self._get_csv_response(self.get_selected_ids()))
        elif export == 'excel':
            raise ComponentHasResponseException(self._get_xls_response(self.get_selected_ids()))

    def _get_csv_response(self, ids=()):
        list_csv = get_metadata(self.qs.model, 'list_csv')
        qs = self.get_queryset(False)
        if ids:
            qs = qs.filter(id__in=ids)
        return CsvResponse(tolist(qs, list_display=list_csv))

    def _get_xls_response(self, ids=()):
        list_xls = get_metadata(self.qs.model, 'list_xls')
        qs = self.get_queryset(False)
        if ids:
            qs = qs.filter(id__in=ids)
        return XlsResponse([('Dados', tolist(qs, list_display=list_xls))])

    def _get_pdf_response(self):
        self.as_pdf = True
        landscape = len(self.list_display) > 4
        return ReportResponse(self.title, self.request, [self], landscape)

    def __str__(self):
        return self.render('base/paginator.html')



# -*- coding: utf-8 -*-
from django.conf import settings
from django.core.management.base import BaseCommand
from djangoplus.admin.models import User


class Command(BaseCommand):
    def handle(self, *args, **options):
        user = User.objects.first()
        user.set_password(settings.DEFAULT_PASSWORD)
        User.objects.update(password=user.password)



# -*- coding: utf-8 -*-

from django.http.response import HttpResponseRedirect


def reset(request, path):
    stack = request.session.get('stack')
    if stack:
        for _ in stack[1:]:
            stack.pop()
        request.session.save()
    return HttpResponseRedirect('/{}'.format(path))



# -*- coding: utf-8 -*-
from tempfile import mktemp
from django.template.loader import render_to_string
from django.http import HttpResponse, HttpResponseRedirect
from django.conf import settings
import tempfile
import json
import os
import datetime
from django.contrib import messages
from djangoplus.utils.formatter import to_ascii


def mobile(request):
    if request:
        for device in ['iphone', 'android']:
            if device in request.META.get('HTTP_USER_AGENT', '').lower():
                return True
    return False

def httprr(request, url, message='', error=False):
    if message:
        if error:
            messages.error(request, message, extra_tags='danger')
        else:
            messages.success(request, message, extra_tags='success')

    if 'popup' in request.GET:
        return HttpResponse(url)

    if url in ('.', '..'):
        back = abs(url == '..' and -1 or 0)
        stack = request.session.get('stack', [])
        if len(stack) >= back:
            while back:
                stack.pop()
                back -= 1
            request.session.save()
            if stack:
                title, url = stack[-1]
        else:
            url = request.get_full_path()
    if request.is_ajax():
        return HttpResponse(url)
    else:
        return HttpResponseRedirect(url)


class XlsResponse(HttpResponse):
    def __init__(self, data, name='Listagem'):
        import xlwt
        wb = xlwt.Workbook(encoding='iso8859-1')
        for title, rows in data:
            sheet = wb.add_sheet(title)
            for row_idx, row in enumerate(rows):
                for col_idx, label in enumerate(row):
                    sheet.write(row_idx, col_idx, label=label)
        file_name = mktemp()
        wb.save(file_name)
        HttpResponse.__init__(self, content=open(file_name, 'rb').read(), content_type='application/vnd.ms-excel')
        self['Content-Disposition'] = 'attachment; filename={}.xls'.format(to_ascii(name))


class CsvResponse(HttpResponse):
    def __init__(self, rows, name='Listagem'):
        import csv
        file_name = mktemp()
        with open(file_name, 'w', encoding='iso8859-1') as output:
            writer = csv.writer(output)
            for row in rows:
                writer.writerow([col for col in row]) # .encode('iso8859-1')
        HttpResponse.__init__(self, content=open(file_name, 'r').read(), content_type='application/csv')
        self['Content-Disposition'] = 'attachment; filename={}.xls'.format(to_ascii(name))


class ZipResponse(HttpResponse):
    def __init__(self, file_path):
        content = open(file_path, 'r+b')
        HttpResponse.__init__(self, content=content, content_type='application/zip')
        self['Content-Disposition'] = 'attachment; filename={}'.format(to_ascii(file_path.split(os.sep)[-1]))


class JsonResponse(HttpResponse):
    def __init__(self, data):
        content = json.dumps(data)
        HttpResponse.__init__(self, content=content, content_type='application/json')


class PdfResponse(HttpResponse):

    def __init__(self, html, landscape=False):
        import pdfkit
        file_name = tempfile.mktemp('.pdf')
        if landscape:
            html = html.replace('logo_if_portrait', 'logo_if_landscape')
            html = html.replace('content="Portrait"', 'content="Landscape"')
        html = html.replace('/media', settings.MEDIA_ROOT)
        html = html.replace('/static', '{}/{}/static'.format(settings.BASE_DIR, settings.PROJECT_NAME))
        pdfkit.from_string(html, file_name)
        #from weasyprint import HTML, CSS
        #HTML(string=html).write_pdf(file_name)
        str_bytes = open(file_name, "rb").read()
        os.unlink(file_name)
        HttpResponse.__init__(self, str_bytes, content_type='application/pdf')


class ReportResponse(PdfResponse):
    def __init__(self, title, request, objects, landscape=False, template='report.html'):
        from djangoplus.admin.models import Settings
        app_settings = Settings.default()
        logo = app_settings.logo_pdf and app_settings.logo_pdf or app_settings.logo
        project_name = app_settings.initials
        project_description = app_settings.name
        unit_or_organization = request.session.get('scope')
        context = dict(
            objects=objects, title=title, today=datetime.date.today(), logo=logo,
            project_name=project_name, project_description=project_description,
            unit_or_organization=unit_or_organization
        )
        html = render_to_string([template], context, request=request)
        super(ReportResponse, self).__init__(html, landscape)


class ComponentResponse(HttpResponse):
    def __init__(self, component):
        super(ComponentResponse, self).__init__(
            render_to_string('default.html', context=dict(paginator=component), request=component.request)
        )


def return_response(f_return, request, title, style, template_name, raise_response=False, ignore_pdf=False):
    from datetime import datetime
    from django.shortcuts import render
    from djangoplus.admin.models import Settings
    from djangoplus.ui.components import ComponentHasResponseException
    if type(f_return) == dict:
        if 'self' in f_return:
            f_return['obj'] = f_return['self']
        if title and 'title' not in f_return:
            f_return['title'] = title
        for key in f_return:
            if hasattr(f_return[key], 'process_request'):
                f_return[key].process_request()
        if 'pdf' in style and not ignore_pdf:
            request.GET._mutable = True
            request.GET['pdf'] = 1
            request.GET._mutable = False
            app_settings = Settings.default()
            f_return['logo'] = app_settings.logo_pdf and app_settings.logo_pdf or app_settings.logo
            f_return['project_name'] = app_settings.initials
            f_return['project_description'] = app_settings.name
            f_return['today'] = datetime.now()
            f_return['default_template'] = 'report.html'
            template_list = [template_name, 'report.html']
            landscape = 'landscape' in style
            response = PdfResponse(render_to_string(template_list, f_return, request=request), landscape=landscape)
        else:
            template_list = [template_name, 'default.html']
            response = render(request, template_list, f_return)
    else:
        response = f_return
    if raise_response:
        raise ComponentHasResponseException(response)
    else:
        return response



# -*- coding: utf-8 -*-
import os
import psycopg2
import sqlite3
from django.conf import settings
from django.core.management import call_command
from django.core.management.base import BaseCommand


class Command(BaseCommand):
    def handle(self, *args, **options):
        dbinfo = settings.DATABASES.get('postgres')
        username, password = dbinfo['USER'], dbinfo['PASSWORD']

        # (re)creating postgres database
        conn = psycopg2.connect('user={} password={}'.format(username, password))
        conn.autocommit = True
        cur = conn.cursor()
        cur.execute('{} {};'.format('drop database if exists', settings.PROJECT_NAME))
        cur.execute('{} {};'.format('create database', settings.PROJECT_NAME))
        cur.close()
        conn.close()

        # creating tables
        call_command('migrate', database='postgres', ignore_sync_permissions=True)

        # reseting permission mapping on admin_users to avoid error on data loading
        conn = sqlite3.connect('sqlite.db')
        cur = conn.cursor()
        cur.execute('update admin_user set permission_mapping = "{}";')
        conn.commit()
        conn.close()

        # deleting content types to avoid error on data loading
        conn = psycopg2.connect('dbname={} user={} password={}'.format(settings.PROJECT_NAME, username, password))
        cur = conn.cursor()
        cur.execute('delete from django_content_type;')
        conn.commit()
        conn.close()

        # dumping data in temporary file
        with open('/tmp/output.json', 'w+') as f:
            call_command('dumpdata', database='sqlite', stdout=f)
            f.close()

        # loading data
        call_command('loaddata', '/tmp/output.json', database='postgres')

        # deleting temporary file
        os.unlink('/tmp/output.json')




# -*- coding: utf-8 -*-

import re
import datetime
import unicodedata
from decimal import Decimal
from django.conf import settings
from collections import Iterable
from django.utils.safestring import mark_safe


def normalyze(nome):
    nome = str(nome)

    if nome.isupper():
        return nome

    ponto = '\.'
    ponto_espaco = '. '
    espaco = ' '
    regex_multiplos_espacos = '\s+'
    regex_numero_romano = '^M{0,4}(CM|CD|D?C{0,3})(XC|XL|L?X{0,3})(IX|IV|V?I{0,3})$'

    nome = re.sub(ponto, ponto_espaco, nome)  # colocando espaço após nomes abreviados
    nome = re.sub(regex_multiplos_espacos, espaco, nome)  # retirando espaços múltiplos
    nome = nome.title()  # alterando os nomes para CamelCase
    partes_nome = nome.split(espaco)  # separando as palavras numa lista
    excecoes = [
        'de', 'di', 'do', 'da', 'dos', 'das', 'dello', 'della', 'dalla',
        'dal', 'del', 'e', 'em', 'na', 'no', 'nas', 'nos', 'van', 'von', 'y', 'para', 'pela', 'pelo', 'por',
    ]

    resultado = []

    for palavra in partes_nome:
        if palavra.lower() in excecoes:
            resultado.append(palavra.lower())
        elif re.match(regex_numero_romano, palavra.upper()):
            resultado.append(palavra.upper())
        else:
            resultado.append(palavra)

    nome = espaco.join(resultado)
    return nome


def format_bool(value):
    return value and '<span class="label label-success">Sim</span>' or '<span class="label label-danger">Não</span>'


def format_value(value, html=True):
    from djangoplus.db.models.fields import ImageFieldFile
    from django.db.models.fields.files import FieldFile, ImageFieldFile as DjangoImageFieldFile
    if 'ManyRelatedManager' in type(value).__name__:
        value = value.all()
    if value in (None, '', ()):
        return '-'
    elif isinstance(value, str) or type(value).__name__ == '__proxy__':  # lazy i18n
        if html:
            return mark_safe(value.replace('\n', '<br>'))
        else:
            return value
    elif isinstance(value, bool):
        return value and 'Sim' or 'Não'
    elif isinstance(value, datetime.datetime):
        return value.strftime('%d/%m/%Y %H:%M')
    elif isinstance(value, datetime.date):
        return value.strftime('%d/%m/%Y')
    elif isinstance(value, tuple):
        return '{} {}'.format(value[0], value[1])
    elif isinstance(value, Decimal):
        if hasattr(value, 'decimal3'):
            return format_decimal3(value)
        elif hasattr(value, 'decimal1'):
            return format_decimal1(value)
        else:
            return format_decimal(value)
    elif isinstance(value, ImageFieldFile) or isinstance(value, DjangoImageFieldFile):
        if html:
            return mark_safe(
                '<img width="75" class="materialboxed" src="{}"/>'.format(value.url)
            )
        else:
            return value.url
    elif isinstance(value, FieldFile):
        file_name = value.name.split('/')[-1]
        if value.url.lower().endswith('.pdf'):
            if html:
                return mark_safe('''
                    <a class="ajax pdf" href="{}">{}</a>{}
                    <a href="{}"><i class="mdi-file-file-download"></i></a>
                '''.format(value.url, file_name, '&nbsp;'*6, value.url))
            else:
                return value.url
        else:
            if html:
                return mark_safe('<a target="_blank" href="{}">{}</a>'.format(value.url, file_name))
            else:
                return value.url
    elif isinstance(value, dict):
        if html:
            ul = ['<ul style="display: inline-block; padding-left:0px">']
            for key, info in value.items():
                ul.append('<li style="list-style-type:none">{}: {}</li>'.format(key, format_value(info, html=html)))
            ul.append('</ul>')
            return mark_safe(''.join(ul))
        else:
            items = []
            for key, info in value.items():
                items.append('{}: {}'.format(key, format_value(info, html=html)))
            return ', '.join(items)
    elif isinstance(value, Iterable):
        if html:
            ul = ['<ul style="display: inline-block; padding-left:20px">']
            for obj in value:
                ul.append('<li style="list-style-type:square">{}</li>'.format(obj))
            ul.append('</ul>')
            return mark_safe(''.join(ul))
        else:
            items = []
            for obj in value:
                items.append(str(obj))
            return ', '.join(items)
    else:
        return str(value)


def format_decimal(value, decimal_places=2):
    str_format = '{{:.{}f}}'.format(decimal_places)
    if value is not None:
        value = str_format.format(Decimal(value))
        if settings.LANGUAGE_CODE == 'pt-br':
            value = value.replace('.', ',')
    return value


def format_decimal3(value):
    if value is None:
        return ''
    return format_decimal(value, 3)


def format_decimal1(value):
    if value is None:
        return ''
    return format_decimal(value, 1)


def to_ascii(txt, codif='utf-8'):
    if not isinstance(txt, str):
        txt = str(txt)
    if isinstance(txt, str):
        txt = txt.encode('utf-8')
    return unicodedata.normalize(
        'NFKD', txt.decode(codif)
    ).encode('ASCII', 'ignore').decode('utf-8')



# -*- coding: utf-8 -*-
import sys
from django.conf import settings
from django.template import loader
from django.core.mail import EmailMultiAlternatives


def send_mail(subject, message, send_to, reply_to=None, actions=()):
    from djangoplus.admin.models import Settings
    url = 'http://{}'.format(settings.HOST_NAME or 'localhost:8000')
    app_settings = Settings.default()
    context = dict()
    context['subject'] = subject
    context['project_url'] = url
    context['project_name'] = app_settings.initials
    context['project_description'] = app_settings.name
    context['project_logo'] = app_settings.logo and \
        '{}/media/{}'.format(url, app_settings.logo) or '{}/static/images/mail.png'.format(url)
    context['actions'] = actions
    context['message'] = message.replace('\n', '<br>').replace('\t', '&nbsp;'*4)
    reply_to = reply_to and [reply_to] or None
    from_email = 'Não-Responder <{}>'.format(settings.SERVER_EMAIL)
    html = loader.render_to_string('mail.html', context)

    if settings.SENDGRID_KEY and 'test' not in sys.argv:
        import sendgrid
        from sendgrid.helpers.mail import Email, Content, Mail

        sg_client = sendgrid.SendGridAPIClient(apikey=settings.SENDGRID_KEY)
        from_email = Email(settings.SERVER_EMAIL)
        to_email = Email(send_to)
        content = Content("text/html", html)
        mail = Mail(from_email, subject, to_email, content)
        if reply_to:
            mail.reply_to = Email(reply_to)
        response = sg_client.client.mail.send.post(request_body=mail.get())
        return response.status_code
    else:
        body = 'Mensagem em anexo.'
        email = EmailMultiAlternatives(subject, body, from_email, [send_to], reply_to=reply_to)
        email.attach_alternative(html, "text/html")
        return email.send()



# Generated by Django 2.0.3 on 2018-04-16 10:58

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import djangoplus.admin.models
import djangoplus.db.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ('admin', '0005_auto_20180416_1045'),
    ]

    operations = [
        migrations.CreateModel(
            name='Role',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
            ],
            options={
                'verbose_name': 'Função',
                'verbose_name_plural': 'Funções',
            },
        ),
        migrations.AddField(
            model_name='role',
            name='group',
            field=djangoplus.db.models.fields.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='admin.Group', verbose_name='Grupo'),
        ),
        migrations.AddField(
            model_name='role',
            name='scope',
            field=djangoplus.db.models.fields.ForeignKey(null=True, blank=True, on_delete=django.db.models.deletion.CASCADE, to='admin.Scope', verbose_name='Scope'),
        ),
        migrations.AddField(
            model_name='role',
            name='user',
            field=djangoplus.db.models.fields.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='Usuário'),
        ),
        migrations.AddField(
            model_name='role',
            name='active',
            field=djangoplus.db.models.fields.BooleanField(default=True, verbose_name='Active'),
        ),
        migrations.RemoveField(
            model_name='user',
            name='organization',
        ),
        migrations.RemoveField(
            model_name='user',
            name='unit',
        ),
    ]



# -*- coding: utf-8 -*-
import uuid
from django.utils.text import slugify
from djangoplus.utils.http import ReportResponse
from django.utils.deprecation import MiddlewareMixin
from django.template.loader import render_to_string
from djangoplus.utils import http
from jinja2.utils import Markup


class Component(object):

    template_name = None
    formatter_name = None

    class Media:
        css = {'all': ()}
        js = ()

    def __init__(self, uid=None, request=None, title=None):
        self.id = slugify(str(uid or uuid.uuid1().hex))
        self.as_pdf = False
        self.title = title
        self.request = request
        self.response = None
        self.mobile = http.mobile(request)

        if self.request:
            if 'pdf' in self.request.GET:
                self.as_pdf = True

    def __str__(self):
        template_name = self.template_name or '{}.html'.format(self.__class__.__name__.lower())
        return self.render(template_name)

    def process_request(self):
        if self.request:
            if self.request.GET.get('pdf') == self.id:
                raise ComponentHasResponseException(
                    ReportResponse(self.title, self.request, [self])
                )

    def render(self, template_name):
        medias = []
        media_cls = getattr(self, 'Media')
        if hasattr(media_cls, 'js'):
            for script in media_cls.js:
                medias.append('<script src="{}"></script>'.format(script))
        if hasattr(media_cls, 'css'):
            for css_media, urls in media_cls.css.items():
                for url in urls:
                    js = '$("<link/>", {{rel: "stylesheet",type: "text/css",href: "{}"}}).appendTo("head");'
                    js = js.format(url)
                    medias.append('<script>{}</script>'.format(js))
        html = render_to_string(template_name, {'component': self})
        return Markup('{}\n{}'.format(''.join(medias), html))

    @classmethod
    def subclasses(cls):
        clss = []

        def find_subclasses(superclass):
            for subclasse in superclass.__subclasses__():
                find_subclasses(subclasse)
            clss.append(superclass)
        find_subclasses(cls)

        return clss


class ComponentHasResponseException(Exception):

    def __init__(self, response):
        self.response = response
        super(ComponentHasResponseException, self).__init__()


class ComponenteResponseMiddleware(MiddlewareMixin):

    def process_exception(self, request, exception):
        if type(exception) == ComponentHasResponseException:
            return exception.response
        return None



# -*- coding: utf-8 -*-
import platform


class Subtitle(object):

    @staticmethod
    def display(message, duration=4):
        import tkinter as tk
        root = tk.Tk()
        root.overrideredirect(True)
        root.wm_attributes("-topmost", True)
        root.attributes('-alpha', 0.8)
        root.configure(background='black')
        lines = list()
        break_line = False
        for i, letter in enumerate(message):
            if break_line or i and i % 50 == 0:
                if letter == ' ':
                    lines.append('\n')
                    break_line = False
                else:
                    break_line = True
            lines.append(letter)
        message = ''.join(lines)
        line_breaks = message.count('\n')
        if 'darwin' in platform.system().lower():
            font_size, width, top, right = 30, 82, int(root.winfo_screenwidth() / 2 - 20 * 35), int(
                root.winfo_screenheight() - [90, 120, 156][line_breaks])
        else:
            font_size, width, top, right = 20, 70, int(root.winfo_screenwidth() / 2 - 70 * 7.5), int(
                root.winfo_screenheight() - [77, 110, 140][line_breaks])
        label = tk.Label(root, text=message, font=("Helvetica", font_size), width=width, height=2 + line_breaks)
        label.configure(foreground="white", background='black')
        label.pack(expand=tk.YES, fill=tk.BOTH)
        root.geometry("+{}+{}".format(top, right))
        root.after(1000 * duration, lambda: root.destroy())
        root.mainloop()



import sys
from collections import OrderedDict
from django.db import connection
from django.core import serializers
from django.db.transaction import atomic

from django.core.management.base import BaseCommand


def update_sequence_value(model):
    last_value = (model.objects.exists() and model.objects.values_list('pk', flat=True).order_by('-id')[0] or 0)
    cursor = connection.cursor()
    cursor.execute('select last_value from %s_id_seq' % model._meta.db_table)
    value = cursor.fetchone()[0]
    if value <= last_value:
        cursor.execute('ALTER SEQUENCE %s_id_seq RESTART WITH %s' % (model._meta.db_table, last_value+1))
        print('Updating sequence for %s to %s' % (model.__name__, last_value+1))


class Command(BaseCommand):

    def add_arguments(self, parser):
        parser.add_argument('models', nargs='*', type=str)
        parser.add_argument('--add', action='store_true', dest='add', default=())

    @atomic
    def handle(self, models, **options):
        data = OrderedDict()
        l = []
        for arg in models:
            data[arg] = []
        s = sys.stdin.read()
        for obj in serializers.deserialize("json", s):
            if obj.object.__class__.__name__ in data:
                data[obj.object.__class__.__name__].append(obj.object)
                if obj.object.__class__ not in l:
                    l.append(obj.object.__class__)

        for key in data:
            print(key, len(data[key]))
            for obj in data[key]:
                obj.save()

        if options.pop('add', False):
            for model in l:
                update_sequence_value(model)